@@ -15,10 +15,15 @@
 
 #include "Document.h"
 
+#include "Align.h"
+
 #include "data/model/WaveFileModel.h"
 #include "data/model/WritableWaveFileModel.h"
 #include "data/model/DenseThreeDimensionalModel.h"
 #include "data/model/DenseTimeValueModel.h"
+#include "data/model/FlexiNoteModel.h"
+#include "data/model/AggregateWaveModel.h"
+
 #include "layer/Layer.h"
 #include "widgets/CommandHistory.h"
 #include "base/Command.h"
@@ -27,16 +32,17 @@
 #include "base/PlayParameters.h"
 #include "transform/TransformFactory.h"
 #include "transform/ModelTransformerFactory.h"
+#include "transform/FeatureExtractionModelTransformer.h"
 #include <QApplication>
 #include <QTextStream>
 #include <QSettings>
 #include <iostream>
 #include <typeinfo>
 
-// For alignment:
-#include "data/model/AggregateWaveModel.h"
-#include "data/model/SparseTimeValueModel.h"
 #include "data/model/AlignmentModel.h"
+#include "Align.h"
+
+using std::vector;
 
 //#define DEBUG_DOCUMENT 1
 
@@ -44,11 +50,21 @@
 
 Document::Document() :
     m_mainModel(0),
-    m_autoAlignment(false)
+    m_autoAlignment(false),
+    m_align(new Align())
 {
-    connect(this, SIGNAL(modelAboutToBeDeleted(Model *)),
+    connect(this,
+            SIGNAL(modelAboutToBeDeleted(Model *)),
             ModelTransformerFactory::getInstance(),
             SLOT(modelAboutToBeDeleted(Model *)));
+
+    connect(ModelTransformerFactory::getInstance(),
+            SIGNAL(transformFailed(QString, QString)),
+            this,
+            SIGNAL(modelGenerationFailed(QString, QString)));
+
+    connect(m_align, SIGNAL(alignmentComplete(AlignmentModel *)),
+            this, SIGNAL(alignmentComplete(AlignmentModel *)));
 }
 
 Document::~Document()
@@ -58,7 +74,7 @@ Document::~Document()
     //the document, be nice to fix that
 
 #ifdef DEBUG_DOCUMENT
-    std::cerr << "\n\nDocument::~Document: about to clear command history" << std::endl;
+    cerr << "\n\nDocument::~Document: about to clear command history" << endl;
 #endif
     CommandHistory::getInstance()->clear();
     
@@ -100,7 +116,6 @@ Document::~Document()
 
     emit mainModelChanged(0);
     delete m_mainModel;
-
 }
 
 Layer *
@@ -139,7 +154,7 @@ Document::createImportedLayer(Model *model)
 	LayerFactory::getInstance()->getValidLayerTypes(model);
 
     if (types.empty()) {
-	std::cerr << "WARNING: Document::importLayer: no valid display layer for model" << std::endl;
+	cerr << "WARNING: Document::importLayer: no valid display layer for model" << endl;
 	return 0;
     }
 
@@ -206,57 +221,192 @@ Document::createDerivedLayer(LayerFactory::LayerType type,
 Layer *
 Document::createDerivedLayer(const Transform &transform,
                              const ModelTransformer::Input &input)
+{
+    Transforms transforms;
+    transforms.push_back(transform);
+    vector<Layer *> layers = createDerivedLayers(transforms, input);
+    if (layers.empty()) return 0;
+    else return layers[0];
+}
+
+vector<Layer *>
+Document::createDerivedLayers(const Transforms &transforms,
+                              const ModelTransformer::Input &input)
 {
     QString message;
-    Model *newModel = addDerivedModel(transform, input, message);
-    if (!newModel) {
-        emit modelGenerationFailed(transform.getIdentifier(), message);
-        return 0;
+    vector<Model *> newModels = addDerivedModels(transforms, input, message, 0);
+
+    if (newModels.empty()) {
+        //!!! This identifier may be wrong!
+        emit modelGenerationFailed(transforms[0].getIdentifier(), message);
+        return vector<Layer *>();
     } else if (message != "") {
-        emit modelGenerationWarning(transform.getIdentifier(), message);
+        //!!! This identifier may be wrong!
+        emit modelGenerationWarning(transforms[0].getIdentifier(), message);
     }
 
-    LayerFactory::LayerTypeSet types =
-	LayerFactory::getInstance()->getValidLayerTypes(newModel);
-
-    if (types.empty()) {
-	std::cerr << "WARNING: Document::createLayerForTransformer: no valid display layer for output of transform " << transform.getIdentifier() << std::endl;
-        newModel->aboutToDelete();
-        emit modelAboutToBeDeleted(newModel);
-        m_models.erase(newModel);
-	delete newModel;
-	return 0;
+    QStringList names;
+    for (int i = 0; i < (int)newModels.size(); ++i) {
+        names.push_back(getUniqueLayerName
+                        (TransformFactory::getInstance()->
+                         getTransformFriendlyName
+                         (transforms[i].getIdentifier())));
     }
 
-    //!!! for now, just use the first suitable layer type
+    vector<Layer *> layers = createLayersForDerivedModels(newModels, names);
+    return layers;
+}
 
-    Layer *newLayer = createLayer(*types.begin());
-    setModel(newLayer, newModel);
+class AdditionalModelConverter : 
+    public ModelTransformerFactory::AdditionalModelHandler
+{
+public:
+    AdditionalModelConverter(Document *doc, 
+                             Document::LayerCreationHandler *handler) :
+        m_doc(doc),
+        m_handler(handler) {
+    }
 
-    //!!! We need to clone the model when adding the layer, so that it
-    //can be edited without affecting other layers that are based on
-    //the same model.  Unfortunately we can't just clone it now,
-    //because it probably hasn't been completed yet -- the transform
-    //runs in the background.  Maybe the transform has to handle
-    //cloning and cacheing models itself.
-    //
-    // Once we do clone models here, of course, we'll have to avoid
-    // leaking them too.
-    //
-    // We want the user to be able to add a model to a second layer
-    // _while it's still being calculated in the first_ and have it
-    // work quickly.  That means we need to put the same physical
-    // model pointer in both layers, so they can't actually be cloned.
+    virtual ~AdditionalModelConverter() { }
+
+    void
+    setPrimaryLayers(vector<Layer *> layers) {
+        m_primary = layers;
+    }
+
+    void
+    moreModelsAvailable(vector<Model *> models) {
+        std::cerr << "AdditionalModelConverter::moreModelsAvailable: " << models.size() << " model(s)" << std::endl;
+        // We can't automatically regenerate the additional models on
+        // reload -- we should delete them instead
+        QStringList names;
+        foreach (Model *model, models) {
+            m_doc->addAdditionalModel(model);
+            names.push_back(QString());
+        }
+        vector<Layer *> layers = m_doc->createLayersForDerivedModels
+            (models, names);
+        m_handler->layersCreated(this, m_primary, layers);
+        delete this;
+    }
+
+    void
+    noMoreModelsAvailable() {
+        std::cerr << "AdditionalModelConverter::noMoreModelsAvailable" << std::endl;
+        m_handler->layersCreated(this, m_primary, vector<Layer *>());
+        delete this;
+    }
+
+    void cancel() {
+        foreach (Layer *layer, m_primary) {
+            Model *model = layer->getModel();
+            if (model) {
+                model->abandon();
+            }
+        }
+    }
+
+private:
+    Document *m_doc;
+    vector<Layer *> m_primary;
+    Document::LayerCreationHandler *m_handler; //!!! how to handle destruction of this?
+};
+
+Document::LayerCreationAsyncHandle
+Document::createDerivedLayersAsync(const Transforms &transforms,
+                                   const ModelTransformer::Input &input,
+                                   LayerCreationHandler *handler)
+{
+    QString message;
+
+    AdditionalModelConverter *amc = new AdditionalModelConverter(this, handler);
     
-    if (newLayer) {
-	newLayer->setObjectName(getUniqueLayerName
-                                (TransformFactory::getInstance()->
-                                 getTransformFriendlyName
-                                 (transform.getIdentifier())));
+    vector<Model *> newModels = addDerivedModels
+        (transforms, input, message, amc);
+
+    QStringList names;
+    for (int i = 0; i < (int)newModels.size(); ++i) {
+        names.push_back(getUniqueLayerName
+                        (TransformFactory::getInstance()->
+                         getTransformFriendlyName
+                         (transforms[i].getIdentifier())));
     }
 
-    emit layerAdded(newLayer);
-    return newLayer;
+    vector<Layer *> layers = createLayersForDerivedModels(newModels, names);
+    amc->setPrimaryLayers(layers);
+
+    if (newModels.empty()) {
+        //!!! This identifier may be wrong!
+        emit modelGenerationFailed(transforms[0].getIdentifier(), message);
+        //!!! what to do with amc?
+    } else if (message != "") {
+        //!!! This identifier may be wrong!
+        emit modelGenerationWarning(transforms[0].getIdentifier(), message);
+        //!!! what to do with amc?
+    }
+
+    return amc;
+}
+
+void
+Document::cancelAsyncLayerCreation(Document::LayerCreationAsyncHandle h)
+{
+    AdditionalModelConverter *conv = static_cast<AdditionalModelConverter *>(h);
+    conv->cancel();
+}
+
+vector<Layer *>
+Document::createLayersForDerivedModels(vector<Model *> newModels, 
+                                       QStringList names)
+{
+    vector<Layer *> layers;
+    
+    for (int i = 0; i < (int)newModels.size(); ++i) {
+
+        Model *newModel = newModels[i];
+
+        LayerFactory::LayerTypeSet types =
+            LayerFactory::getInstance()->getValidLayerTypes(newModel);
+
+        if (types.empty()) {
+            cerr << "WARNING: Document::createLayerForTransformer: no valid display layer for output of transform " << names[i] << endl;
+            //!!! inadequate cleanup:
+            newModel->aboutToDelete();
+            emit modelAboutToBeDeleted(newModel);
+            m_models.erase(newModel);
+            delete newModel;
+            return vector<Layer *>();
+        }
+
+        //!!! for now, just use the first suitable layer type
+
+        Layer *newLayer = createLayer(*types.begin());
+        setModel(newLayer, newModel);
+
+        //!!! We need to clone the model when adding the layer, so that it
+        //can be edited without affecting other layers that are based on
+        //the same model.  Unfortunately we can't just clone it now,
+        //because it probably hasn't been completed yet -- the transform
+        //runs in the background.  Maybe the transform has to handle
+        //cloning and cacheing models itself.
+        //
+        // Once we do clone models here, of course, we'll have to avoid
+        // leaking them too.
+        //
+        // We want the user to be able to add a model to a second layer
+        // _while it's still being calculated in the first_ and have it
+        // work quickly.  That means we need to put the same physical
+        // model pointer in both layers, so they can't actually be cloned.
+    
+        if (newLayer) {
+            newLayer->setObjectName(names[i]);
+        }
+
+        emit layerAdded(newLayer);
+        layers.push_back(newLayer);
+    }
+
+    return layers;
 }
 
 void
@@ -282,14 +432,14 @@ Document::setMainModel(WaveFileModel *model)
     // delete any of the models.
 
 #ifdef DEBUG_DOCUMENT
-    std::cerr << "Document::setMainModel: Have "
-              << m_layers.size() << " layers" << std::endl;
-    std::cerr << "Models now: ";
+    cerr << "Document::setMainModel: Have "
+              << m_layers.size() << " layers" << endl;
+    cerr << "Models now: ";
     for (ModelMap::const_iterator i = m_models.begin(); i != m_models.end(); ++i) {
-        std::cerr << i->first << " ";
+        cerr << i->first << " ";
     } 
-    std::cerr << std::endl;
-    std::cerr << "Old main model: " << oldMainModel << std::endl;
+    cerr << endl;
+    cerr << "Old main model: " << oldMainModel << endl;
 #endif
 
     for (LayerSet::iterator i = m_layers.begin(); i != m_layers.end(); ++i) {
@@ -298,30 +448,30 @@ Document::setMainModel(WaveFileModel *model)
 	Model *model = layer->getModel();
 
 #ifdef DEBUG_DOCUMENT
-        std::cerr << "Document::setMainModel: inspecting model "
+        cerr << "Document::setMainModel: inspecting model "
                   << (model ? model->objectName(): "(null)") << " in layer "
-                  << layer->objectName() << std::endl;
+                  << layer->objectName() << endl;
 #endif
 
 	if (model == oldMainModel) {
 #ifdef DEBUG_DOCUMENT
-            std::cerr << "... it uses the old main model, replacing" << std::endl;
+            cerr << "... it uses the old main model, replacing" << endl;
 #endif
 	    LayerFactory::getInstance()->setModel(layer, m_mainModel);
 	    continue;
 	}
 
         if (!model) {
-            std::cerr << "WARNING: Document::setMainModel: Null model in layer "
-                      << layer << std::endl;
+            cerr << "WARNING: Document::setMainModel: Null model in layer "
+                      << layer << endl;
 	    // get rid of this hideous degenerate
 	    obsoleteLayers.push_back(layer);
 	    continue;
 	}
 
 	if (m_models.find(model) == m_models.end()) {
-	    std::cerr << "WARNING: Document::setMainModel: Unknown model "
-		      << model << " in layer " << layer << std::endl;
+	    cerr << "WARNING: Document::setMainModel: Unknown model "
+		      << model << " in layer " << layer << endl;
 	    // and this one
 	    obsoleteLayers.push_back(layer);
 	    continue;
@@ -331,7 +481,7 @@ Document::setMainModel(WaveFileModel *model)
             (m_models[model].source == oldMainModel)) {
 
 #ifdef DEBUG_DOCUMENT
-            std::cerr << "... it uses a model derived from the old main model, regenerating" << std::endl;
+            cerr << "... it uses a model derived from the old main model, regenerating" << endl;
 #endif
 
 	    // This model was derived from the previous main
@@ -351,8 +501,8 @@ Document::setMainModel(WaveFileModel *model)
                                 message);
 	    
 	    if (!replacementModel) {
-		std::cerr << "WARNING: Document::setMainModel: Failed to regenerate model for transform \""
-			  << transformId << "\"" << " in layer " << layer << std::endl;
+		cerr << "WARNING: Document::setMainModel: Failed to regenerate model for transform \""
+			  << transformId << "\"" << " in layer " << layer << endl;
                 if (failedTransformers.find(transformId)
                     == failedTransformers.end()) {
                     emit modelRegenerationFailed(layer->objectName(),
@@ -368,20 +518,19 @@ Document::setMainModel(WaveFileModel *model)
                                                   message);
                 }
 #ifdef DEBUG_DOCUMENT
-                std::cerr << "Replacing model " << model << " (type "
+                cerr << "Replacing model " << model << " (type "
                           << typeid(*model).name() << ") with model "
                           << replacementModel << " (type "
                           << typeid(*replacementModel).name() << ") in layer "
                           << layer << " (name " << layer->objectName() << ")"
-                          << std::endl;
-#endif
+                          << endl;
+
                 RangeSummarisableTimeValueModel *rm =
                     dynamic_cast<RangeSummarisableTimeValueModel *>(replacementModel);
-#ifdef DEBUG_DOCUMENT
                 if (rm) {
-                    std::cerr << "new model has " << rm->getChannelCount() << " channels " << std::endl;
+                    cerr << "new model has " << rm->getChannelCount() << " channels " << endl;
                 } else {
-                    std::cerr << "new model " << replacementModel << " is not a RangeSummarisableTimeValueModel!" << std::endl;
+                    cerr << "new model " << replacementModel << " is not a RangeSummarisableTimeValueModel!" << endl;
                 }
 #endif
 		setModel(layer, replacementModel);
@@ -393,6 +542,15 @@ Document::setMainModel(WaveFileModel *model)
 	deleteLayer(obsoleteLayers[k], true);
     }
 
+    for (ModelMap::iterator i = m_models.begin(); i != m_models.end(); ++i) {
+        if (i->second.additional) {
+            Model *m = i->first;
+            m->aboutToDelete();
+            emit modelAboutToBeDeleted(m);
+            delete m;
+        }
+    }
+
     for (ModelMap::iterator i = m_models.begin(); i != m_models.end(); ++i) {
 
         Model *m = i->first;
@@ -419,6 +577,7 @@ Document::setMainModel(WaveFileModel *model)
     }
 
     if (m_autoAlignment) {
+        SVDEBUG << "Document::setMainModel: auto-alignment is on, aligning model if possible" << endl;
         alignModel(m_mainModel);
     }
 
@@ -428,21 +587,21 @@ Document::setMainModel(WaveFileModel *model)
 }
 
 void
-Document::addDerivedModel(const Transform &transform,
-                          const ModelTransformer::Input &input,
-                          Model *outputModelToAdd)
+Document::addAlreadyDerivedModel(const Transform &transform,
+                                 const ModelTransformer::Input &input,
+                                 Model *outputModelToAdd)
 {
     if (m_models.find(outputModelToAdd) != m_models.end()) {
-	std::cerr << "WARNING: Document::addDerivedModel: Model already added"
-		  << std::endl;
+	cerr << "WARNING: Document::addAlreadyDerivedModel: Model already added"
+		  << endl;
 	return;
     }
 
 #ifdef DEBUG_DOCUMENT
     if (input.getModel()) {
-        std::cerr << "Document::addDerivedModel: source is " << input.getModel() << " \"" << input.getModel()->objectName() << "\"" << std::endl;
+        cerr << "Document::addAlreadyDerivedModel: source is " << input.getModel() << " \"" << input.getModel()->objectName() << "\"" << endl;
     } else {
-        std::cerr << "Document::addDerivedModel: source is " << input.getModel() << std::endl;
+        cerr << "Document::addAlreadyDerivedModel: source is " << input.getModel() << endl;
     }
 #endif
 
@@ -450,6 +609,7 @@ Document::addDerivedModel(const Transform &transform,
     rec.source = input.getModel();
     rec.channel = input.getChannel();
     rec.transform = transform;
+    rec.additional = false;
     rec.refcount = 0;
 
     outputModelToAdd->setSourceModel(input.getModel());
@@ -457,12 +617,12 @@ Document::addDerivedModel(const Transform &transform,
     m_models[outputModelToAdd] = rec;
 
 #ifdef DEBUG_DOCUMENT
-    SVDEBUG << "Document::addDerivedModel: Added model " << outputModelToAdd << endl;
-    std::cerr << "Models now: ";
+    cerr << "Document::addAlreadyDerivedModel: Added model " << outputModelToAdd << endl;
+    cerr << "Models now: ";
     for (ModelMap::const_iterator i = m_models.begin(); i != m_models.end(); ++i) {
-        std::cerr << i->first << " ";
+        cerr << i->first << " ";
     } 
-    std::cerr << std::endl;
+    cerr << endl;
 #endif
 
     emit modelAdded(outputModelToAdd);
@@ -473,70 +633,150 @@ void
 Document::addImportedModel(Model *model)
 {
     if (m_models.find(model) != m_models.end()) {
-	std::cerr << "WARNING: Document::addImportedModel: Model already added"
-		  << std::endl;
+	cerr << "WARNING: Document::addImportedModel: Model already added"
+		  << endl;
 	return;
     }
 
     ModelRecord rec;
     rec.source = 0;
+    rec.channel = 0;
     rec.refcount = 0;
+    rec.additional = false;
 
     m_models[model] = rec;
 
 #ifdef DEBUG_DOCUMENT
     SVDEBUG << "Document::addImportedModel: Added model " << model << endl;
-    std::cerr << "Models now: ";
+    cerr << "Models now: ";
     for (ModelMap::const_iterator i = m_models.begin(); i != m_models.end(); ++i) {
-        std::cerr << i->first << " ";
+        cerr << i->first << " ";
     } 
-    std::cerr << std::endl;
+    cerr << endl;
 #endif
 
-    if (m_autoAlignment) alignModel(model);
+    if (m_autoAlignment) {
+        SVDEBUG << "Document::addImportedModel: auto-alignment is on, aligning model if possible" << endl;
+        alignModel(model);
+    } else {
+        SVDEBUG << "Document(" << this << "): addImportedModel: auto-alignment is off" << endl;
+    }
 
     emit modelAdded(model);
 }
 
+void
+Document::addAdditionalModel(Model *model)
+{
+    if (m_models.find(model) != m_models.end()) {
+	cerr << "WARNING: Document::addAdditionalModel: Model already added"
+		  << endl;
+	return;
+    }
+
+    ModelRecord rec;
+    rec.source = 0;
+    rec.channel = 0;
+    rec.refcount = 0;
+    rec.additional = true;
+
+    m_models[model] = rec;
+
+#ifdef DEBUG_DOCUMENT
+    SVDEBUG << "Document::addAdditionalModel: Added model " << model << endl;
+    cerr << "Models now: ";
+    for (ModelMap::const_iterator i = m_models.begin(); i != m_models.end(); ++i) {
+        cerr << i->first << " ";
+    } 
+    cerr << endl;
+#endif
+
+    if (m_autoAlignment) {
+        SVDEBUG << "Document::addAdditionalModel: auto-alignment is on, aligning model if possible" << endl;
+        alignModel(model);
+    }
+
+    emit modelAdded(model);
+}
+
+void
+Document::addAggregateModel(AggregateWaveModel *model)
+{
+    connect(model, SIGNAL(modelInvalidated()),
+            this, SLOT(aggregateModelInvalidated()));
+    m_aggregateModels.insert(model);
+}
+
+void
+Document::aggregateModelInvalidated()
+{
+    QObject *s = sender();
+    AggregateWaveModel *aggregate = qobject_cast<AggregateWaveModel *>(s);
+    if (aggregate) releaseModel(aggregate);
+}
+
 Model *
 Document::addDerivedModel(const Transform &transform,
                           const ModelTransformer::Input &input,
                           QString &message)
 {
-    Model *model = 0;
-
     for (ModelMap::iterator i = m_models.begin(); i != m_models.end(); ++i) {
-	if (i->second.transform == transform &&
-	    i->second.source == input.getModel() && 
+        if (i->second.transform == transform &&
+            i->second.source == input.getModel() && 
             i->second.channel == input.getChannel()) {
-	    return i->first;
-	}
+            std::cerr << "derived model taken from map " << std::endl;
+            return i->first;
+        }
     }
 
-    model = ModelTransformerFactory::getInstance()->transform
-        (transform, input, message);
+    Transforms tt;
+    tt.push_back(transform);
+    vector<Model *> mm = addDerivedModels(tt, input, message, 0);
+    if (mm.empty()) return 0;
+    else return mm[0];
+}
 
-    // The transform we actually used was presumably identical to the
-    // one asked for, except that the version of the plugin may
-    // differ.  It's possible that the returned message contains a
-    // warning about this; that doesn't concern us here, but we do
-    // need to ensure that the transform we remember is correct for
-    // what was actually applied, with the current plugin version.
+vector<Model *>
+Document::addDerivedModels(const Transforms &transforms,
+                           const ModelTransformer::Input &input,
+                           QString &message,
+                           AdditionalModelConverter *amc)
+{
+    vector<Model *> mm = 
+        ModelTransformerFactory::getInstance()->transformMultiple
+        (transforms, input, message, amc);
 
-    Transform applied = transform;
-    applied.setPluginVersion
-        (TransformFactory::getInstance()->
-         getDefaultTransformFor(transform.getIdentifier(),
-                                lrintf(transform.getSampleRate()))
-         .getPluginVersion());
+    for (int j = 0; j < (int)mm.size(); ++j) {
 
-    if (!model) {
-	std::cerr << "WARNING: Document::addDerivedModel: no output model for transform " << transform.getIdentifier() << std::endl;
-    } else {
-	addDerivedModel(applied, input, model);
+        Model *model = mm[j];
+
+        // The transform we actually used was presumably identical to
+        // the one asked for, except that the version of the plugin
+        // may differ.  It's possible that the returned message
+        // contains a warning about this; that doesn't concern us
+        // here, but we do need to ensure that the transform we
+        // remember is correct for what was actually applied, with the
+        // current plugin version.
+
+        //!!! would be nice to short-circuit this -- the version is
+        //!!! static data, shouldn't have to construct a plugin for it
+        //!!! (which may be expensive in Piper-world)
+        
+        Transform applied = transforms[j];
+        applied.setPluginVersion
+            (TransformFactory::getInstance()->
+             getDefaultTransformFor(applied.getIdentifier(),
+                                    applied.getSampleRate())
+             .getPluginVersion());
+
+        if (!model) {
+            cerr << "WARNING: Document::addDerivedModel: no output model for transform " << applied.getIdentifier() << endl;
+        } else {
+            addAlreadyDerivedModel(applied, input, model);
+        }
     }
-
-    return model;
+	
+    return mm;
 }
 
 void
@@ -553,18 +793,20 @@ Document::releaseModel(Model *model) // Will _not_ release main model!
     bool toDelete = false;
 
     if (m_models.find(model) != m_models.end()) {
-	
 	if (m_models[model].refcount == 0) {
-	    std::cerr << "WARNING: Document::releaseModel: model " << model
-		      << " reference count is zero already!" << std::endl;
+	    SVCERR << "WARNING: Document::releaseModel: model " << model
+                   << " reference count is zero already!" << endl;
 	} else {
 	    if (--m_models[model].refcount == 0) {
 		toDelete = true;
 	    }
 	}
+    } else if (m_aggregateModels.find(model) != m_aggregateModels.end()) {
+        SVDEBUG << "Document::releaseModel: is an aggregate model" << endl;
+        toDelete = true;
     } else { 
-	std::cerr << "WARNING: Document::releaseModel: Unfound model "
-		  << model << std::endl;
+	SVCERR << "WARNING: Document::releaseModel: Unfound model "
+               << model << endl;
 	toDelete = true;
     }
 
@@ -581,9 +823,9 @@ Document::releaseModel(Model *model) // Will _not_ release main model!
 
 	if (sourceCount > 0) {
 	    SVDEBUG << "Document::releaseModel: Deleting model "
-		      << model << " even though it is source for "
-		      << sourceCount << " other derived model(s) -- resetting "
-		      << "their source fields appropriately" << endl;
+                    << model << " even though it is source for "
+                    << sourceCount << " other derived model(s) -- resetting "
+                    << "their source fields appropriately" << endl;
 	}
 
         model->aboutToDelete();
@@ -592,11 +834,11 @@ Document::releaseModel(Model *model) // Will _not_ release main model!
 
 #ifdef DEBUG_DOCUMENT
         SVDEBUG << "Document::releaseModel: Deleted model " << model << endl;
-        std::cerr << "Models now: ";
+        cerr << "Models now: ";
         for (ModelMap::const_iterator i = m_models.begin(); i != m_models.end(); ++i) {
-            std::cerr << i->first << " ";
+            cerr << i->first << " ";
         } 
-        std::cerr << std::endl;
+        cerr << endl;
 #endif
 
 	delete model;
@@ -609,15 +851,15 @@ Document::deleteLayer(Layer *layer, bool force)
     if (m_layerViewMap.find(layer) != m_layerViewMap.end() &&
 	m_layerViewMap[layer].size() > 0) {
 
-	std::cerr << "WARNING: Document::deleteLayer: Layer "
+	cerr << "WARNING: Document::deleteLayer: Layer "
 		  << layer << " [" << layer->objectName() << "]"
 		  << " is still used in " << m_layerViewMap[layer].size()
-		  << " views!" << std::endl;
+		  << " views!" << endl;
 
 	if (force) {
 
 #ifdef DEBUG_DOCUMENT
-	    std::cerr << "(force flag set -- deleting from all views)" << std::endl;
+	    cerr << "(force flag set -- deleting from all views)" << endl;
 #endif
 
 	    for (std::set<View *>::iterator j = m_layerViewMap[layer].begin();
@@ -661,11 +903,11 @@ Document::setModel(Layer *layer, Model *model)
     if (model && 
 	model != m_mainModel &&
 	m_models.find(model) == m_models.end()) {
-	std::cerr << "ERROR: Document::setModel: Layer " << layer
-		  << " (\"" << layer->objectName().toStdString()
+	cerr << "ERROR: Document::setModel: Layer " << layer
+		  << " (\"" << layer->objectName()
                   << "\") wants to use unregistered model " << model
 		  << ": register the layer's model before setting it!"
-		  << std::endl;
+		  << endl;
 	return;
     }
 
@@ -690,6 +932,7 @@ Document::setModel(Layer *layer, Model *model)
     }
 
     LayerFactory::getInstance()->setModel(layer, model);
+	// std::cerr << "layer type: " << LayerFactory::getInstance()->getLayerTypeName(LayerFactory::getInstance()->getLayerType(layer)) << std::endl;
 
     if (previousModel) {
         releaseModel(previousModel);
@@ -715,9 +958,9 @@ Document::addLayerToView(View *view, Layer *layer)
     } else {
 	if (model != m_mainModel &&
 	    m_models.find(model) == m_models.end()) {
-	    std::cerr << "ERROR: Document::addLayerToView: Layer " << layer
+	    cerr << "ERROR: Document::addLayerToView: Layer " << layer
 		      << " has unregistered model " << model
-		      << " -- register the layer's model before adding the layer!" << std::endl;
+		      << " -- register the layer's model before adding the layer!" << endl;
 	    return;
 	}
     }
@@ -741,9 +984,9 @@ Document::addToLayerViewMap(Layer *layer, View *view)
 
     if (m_layerViewMap[layer].find(view) !=
 	m_layerViewMap[layer].end()) {
-	std::cerr << "WARNING: Document::addToLayerViewMap:"
+	cerr << "WARNING: Document::addToLayerViewMap:"
 		  << " Layer " << layer << " -> view " << view << " already in"
-		  << " layer view map -- internal inconsistency" << std::endl;
+		  << " layer view map -- internal inconsistency" << endl;
     }
 
     m_layerViewMap[layer].insert(view);
@@ -756,9 +999,9 @@ Document::removeFromLayerViewMap(Layer *layer, View *view)
 {
     if (m_layerViewMap[layer].find(view) ==
 	m_layerViewMap[layer].end()) {
-	std::cerr << "WARNING: Document::removeFromLayerViewMap:"
+	cerr << "WARNING: Document::removeFromLayerViewMap:"
 		  << " Layer " << layer << " -> view " << view << " not in"
-		  << " layer view map -- internal inconsistency" << std::endl;
+		  << " layer view map -- internal inconsistency" << endl;
     }
 
     m_layerViewMap[layer].erase(view);
@@ -823,37 +1066,31 @@ Document::isKnownModel(const Model *model) const
     return (m_models.find(const_cast<Model *>(model)) != m_models.end());
 }
 
-TransformId
-Document::getAlignmentTransformName()
-{
-    QSettings settings;
-    settings.beginGroup("Alignment");
-    TransformId id =
-        settings.value("transform-id",
-                       "vamp:match-vamp-plugin:match:path").toString();
-    settings.endGroup();
-    return id;
-}
-
 bool
-Document::canAlign() 
+Document::canAlign()
 {
-    TransformId id = getAlignmentTransformName();
-    TransformFactory *factory = TransformFactory::getInstance();
-    return factory->haveTransform(id);
+    return Align::canAlign();
 }
 
 void
 Document::alignModel(Model *model)
 {
-    if (!m_mainModel) return;
+    SVDEBUG << "Document::alignModel(" << model << ")" << endl;
+
+    if (!m_mainModel) {
+        SVDEBUG << "(no main model to align to)" << endl;
+        return;
+    }
 
     RangeSummarisableTimeValueModel *rm = 
         dynamic_cast<RangeSummarisableTimeValueModel *>(model);
-    if (!rm) return;
+    if (!rm) {
+        SVDEBUG << "(main model is not alignable-to)" << endl;
+        return;
+    }
 
     if (rm->getAlignmentReference() == m_mainModel) {
-        SVDEBUG << "Document::alignModel: model " << rm << " is already aligned to main model " << m_mainModel << endl;
+        SVDEBUG << "(model " << rm << " is already aligned to main model " << m_mainModel << ")" << endl;
         return;
     }
     
@@ -867,75 +1104,10 @@ Document::alignModel(Model *model)
         return;
     }
 
-    // This involves creating three new models:
-
-    // 1. an AggregateWaveModel to provide the mixdowns of the main
-    // model and the new model in its two channels, as input to the
-    // MATCH plugin
-
-    // 2. a SparseTimeValueModel, which is the model automatically
-    // created by FeatureExtractionPluginTransformer when running the
-    // MATCH plugin (thus containing the alignment path)
-
-    // 3. an AlignmentModel, which stores the path model and carries
-    // out alignment lookups on it.
-
-    // The first two of these are provided as arguments to the
-    // constructor for the third, which takes responsibility for
-    // deleting them.  The AlignmentModel, meanwhile, is passed to the
-    // new model we are aligning, which also takes responsibility for
-    // it.  We should not have to delete any of these new models here.
-
-    AggregateWaveModel::ChannelSpecList components;
-
-    components.push_back(AggregateWaveModel::ModelChannelSpec
-                         (m_mainModel, -1));
-
-    components.push_back(AggregateWaveModel::ModelChannelSpec
-                         (rm, -1));
-
-    Model *aggregateModel = new AggregateWaveModel(components);
-    ModelTransformer::Input aggregate(aggregateModel);
-
-    TransformId id = "vamp:match-vamp-plugin:match:path"; //!!! configure
-    
-    TransformFactory *tf = TransformFactory::getInstance();
-
-    Transform transform = tf->getDefaultTransformFor
-        (id, aggregateModel->getSampleRate());
-
-    transform.setStepSize(transform.getBlockSize()/2);
-    transform.setParameter("serialise", 1);
-
-    SVDEBUG << "Document::alignModel: Alignment transform step size " << transform.getStepSize() << ", block size " << transform.getBlockSize() << endl;
-
-    ModelTransformerFactory *mtf = ModelTransformerFactory::getInstance();
-
-    QString message;
-    Model *transformOutput = mtf->transform(transform, aggregate, message);
-
-    if (!transformOutput) {
-        transform.setStepSize(0);
-        transformOutput = mtf->transform(transform, aggregate, message);
+    if (!m_align->alignModel(m_mainModel, rm)) {
+        cerr << "Alignment failed: " << m_align->getError() << endl;
+        emit alignmentFailed(m_align->getError());
     }
-
-    SparseTimeValueModel *path = dynamic_cast<SparseTimeValueModel *>
-        (transformOutput);
-
-    if (!path) {
-        std::cerr << "Document::alignModel: ERROR: Failed to create alignment path (no MATCH plugin?)" << std::endl;
-        emit alignmentFailed(id, message);
-        delete transformOutput;
-        delete aggregateModel;
-        return;
-    }
-
-    path->setCompletion(0);
-
-    AlignmentModel *alignmentModel = new AlignmentModel
-        (m_mainModel, model, aggregateModel, path);
-
-    rm->setAlignment(alignmentModel);
 }
 
 void
@@ -1013,6 +1185,7 @@ Document::RemoveLayerCommand::RemoveLayerCommand(Document *d,
     m_d(d),
     m_view(view),
     m_layer(layer),
+    m_wasDormant(layer->isLayerDormant(view)),
     m_name(qApp->translate("RemoveLayerCommand", "Delete %1 Layer").arg(layer->objectName())),
     m_added(true)
 {
@@ -1066,7 +1239,7 @@ void
 Document::RemoveLayerCommand::unexecute()
 {
     m_view->addLayer(m_layer);
-    m_layer->setLayerDormant(m_view, false);
+    m_layer->setLayerDormant(m_view, m_wasDormant);
 
     m_d->addToLayerViewMap(m_layer, m_view);
     m_added = true;
@@ -1117,11 +1290,38 @@ Document::toXml(QTextStream &out, QString indent, QString extraAttributes,
     for (LayerViewMap::const_iterator i = m_layerViewMap.begin();
          i != m_layerViewMap.end(); ++i) {
 
-        if (i->first && !i->second.empty() && i->first->getModel()) {
-            used.insert(i->first->getModel());
+        if (i->first && !i->second.empty()) { // Layer exists, is in views
+            Model *m = i->first->getModel();
+            if (m) {
+                used.insert(m);
+                if (m->getSourceModel()) {
+                    used.insert(m->getSourceModel());
+                }
+            }
         }
     }
 
+    // Write aggregate models first, so that when re-reading
+    // derivations we already know about their existence. But only
+    // those that are actually used
+
+    for (std::set<Model *>::iterator i = m_aggregateModels.begin();
+         i != m_aggregateModels.end(); ++i) {
+
+        SVDEBUG << "checking aggregate model " << *i << endl;
+        
+        AggregateWaveModel *aggregate = qobject_cast<AggregateWaveModel *>(*i);
+        if (!aggregate) continue; 
+        if (used.find(aggregate) == used.end()) {
+            SVDEBUG << "(unused, skipping)" << endl;
+            continue;
+        }
+
+        SVDEBUG << "(used, writing)" << endl;
+
+        aggregate->toXml(out, indent + "  ");
+    }
+
     std::set<Model *> written;
 
     for (ModelMap::const_iterator i = m_models.begin();
@@ -1180,7 +1380,7 @@ Document::toXml(QTextStream &out, QString indent, QString extraAttributes,
                  .arg(XmlExportable::getObjectExportId(model)));
         }
     }
-	    
+    
     //!!!
 
     // We should write out the alignment models here.  AlignmentModel
