@@ -19,26 +19,34 @@
 #include <QObject>
 #include <QString>
 
+#include "base/DataExportOptions.h"
+
 class Model;
+class MultiSelection;
 
 class CSVFileWriter : public QObject
 {
     Q_OBJECT
 
 public:
-    CSVFileWriter(QString path, Model *model, QString delimiter = ",");
+    CSVFileWriter(QString path,
+                  Model *model,
+                  QString delimiter = ",",
+                  DataExportOptions options = DataExportDefaults);
     virtual ~CSVFileWriter();
 
     virtual bool isOK() const;
     virtual QString getError() const;
 
     virtual void write();
+    virtual void writeSelection(MultiSelection *selection);
 
 protected:
     QString m_path;
     Model *m_model;
     QString m_error;
     QString m_delimiter;
+    DataExportOptions m_options;
 };
 
 #endif
