@@ -25,12 +25,11 @@
 #include <QFileInfo>
 #include <QDir>
 #include <QCoreApplication>
+#include <QThreadStorage>
 
 #include <iostream>
 #include <cstdlib>
 
-#include <unistd.h>
-
 //#define DEBUG_FILE_SOURCE 1
 
 int
@@ -50,32 +49,53 @@ FileSource::m_mapMutex;
 
 #ifdef DEBUG_FILE_SOURCE
 static int extantCount = 0;
+static int threadCount = 0;
 static std::map<QString, int> urlExtantCountMap;
+static QMutex countMutex;
 static void incCount(QString url) {
+    QMutexLocker locker(&countMutex);
     ++extantCount;
     if (urlExtantCountMap.find(url) == urlExtantCountMap.end()) {
         urlExtantCountMap[url] = 1;
     } else {
         ++urlExtantCountMap[url];
     }
-    std::cerr << "FileSource: Now " << urlExtantCountMap[url] << " for this url, " << extantCount << " total" << std::endl;
+    cerr << "FileSource: Now " << urlExtantCountMap[url] << " for this url, " << extantCount << " total" << endl;
 }
 static void decCount(QString url) {
+    QMutexLocker locker(&countMutex);
     --extantCount;
     --urlExtantCountMap[url];
-    std::cerr << "FileSource: Now " << urlExtantCountMap[url] << " for this url, " << extantCount << " total" << std::endl;
+    cerr << "FileSource: Now " << urlExtantCountMap[url] << " for this url, " << extantCount << " total" << endl;
 }
+void
+FileSource::debugReport()
+{
+    QMutexLocker locker(&countMutex);
+    cerr << "\nFileSource::debugReport: Have " << extantCount << " FileSource object(s) extant across " << threadCount << " thread(s)" << endl;
+    cerr << "URLs by extant count:" << endl;
+    cerr << "Count | URL" << endl;
+    for (std::map<QString, int>::const_iterator i = urlExtantCountMap.begin();
+         i != urlExtantCountMap.end(); ++i) {
+        cerr << i->second << " | " << i->first << endl;
+    }
+    cerr << "FileSource::debugReport done\n" << endl;
+}
+#else
+void FileSource::debugReport() { }
 #endif
 
-static QNetworkAccessManager nm;
+static QThreadStorage<QNetworkAccessManager *> nms;
 
 FileSource::FileSource(QString fileOrUrl, ProgressReporter *reporter,
                        QString preferredContentType) :
+    m_rawFileOrUrl(fileOrUrl),
     m_url(fileOrUrl, QUrl::StrictMode),
     m_localFile(0),
     m_reply(0),
     m_preferredContentType(preferredContentType),
     m_ok(false),
+    m_cancelled(false),
     m_lastStatus(0),
     m_resource(fileOrUrl.startsWith(':')),
     m_remote(isRemote(fileOrUrl)),
@@ -93,12 +113,12 @@ FileSource::FileSource(QString fileOrUrl, ProgressReporter *reporter,
     }
  
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource::FileSource(" << fileOrUrl << "): url <" << m_url.toString() << ">" << std::endl;
+    cerr << "FileSource::FileSource(" << fileOrUrl << "): url <" << m_url.toString() << ">" << endl;
     incCount(m_url.toString());
 #endif
 
     if (!canHandleScheme(m_url)) {
-        std::cerr << "FileSource::FileSource: ERROR: Unsupported scheme in URL \"" << m_url.toString() << "\"" << std::endl;
+        cerr << "FileSource::FileSource: ERROR: Unsupported scheme in URL \"" << m_url.toString() << "\"" << endl;
         m_errorString = tr("Unsupported scheme in URL");
         return;
     }
@@ -108,9 +128,12 @@ FileSource::FileSource(QString fileOrUrl, ProgressReporter *reporter,
     if (!isRemote() &&
         !isAvailable()) {
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::FileSource: Failed to open local file with URL \"" << m_url.toString() << "\"; trying again assuming filename was encoded" << std::endl;
+        cerr << "FileSource::FileSource: Failed to open local file with URL \"" << m_url.toString() << "\"; trying again assuming filename was encoded" << endl;
 #endif
         m_url = QUrl::fromEncoded(fileOrUrl.toLatin1());
+#ifdef DEBUG_FILE_SOURCE
+        cerr << "FileSource::FileSource: URL is now \"" << m_url.toString() << "\"" << endl;
+#endif
         init();
     }
 
@@ -125,11 +148,11 @@ FileSource::FileSource(QString fileOrUrl, ProgressReporter *reporter,
             // The URL was created on the assumption that the string
             // was human-readable.  Let's try again, this time
             // assuming it was already encoded.
-            std::cerr << "FileSource::FileSource: Failed to retrieve URL \""
-                      << fileOrUrl.toStdString() 
+            cerr << "FileSource::FileSource: Failed to retrieve URL \""
+                      << fileOrUrl 
                       << "\" as human-readable URL; "
                       << "trying again treating it as encoded URL"
-                      << std::endl;
+                      << endl;
 
             // even though our cache file doesn't exist (because the
             // resource was 404), we still need to ensure we're no
@@ -153,7 +176,7 @@ FileSource::FileSource(QString fileOrUrl, ProgressReporter *reporter,
     }
 
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource::FileSource(string) exiting" << std::endl;
+    cerr << "FileSource::FileSource(string) exiting" << endl;
 #endif
 }
 
@@ -162,6 +185,7 @@ FileSource::FileSource(QUrl url, ProgressReporter *reporter) :
     m_localFile(0),
     m_reply(0),
     m_ok(false),
+    m_cancelled(false),
     m_lastStatus(0),
     m_resource(false),
     m_remote(isRemote(url.toString())),
@@ -171,12 +195,12 @@ FileSource::FileSource(QUrl url, ProgressReporter *reporter) :
     m_refCounted(false)
 {
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource::FileSource(" << url.toString() << ") [as url]" << std::endl;
+    cerr << "FileSource::FileSource(" << url.toString() << ") [as url]" << endl;
     incCount(m_url.toString());
 #endif
 
     if (!canHandleScheme(m_url)) {
-        std::cerr << "FileSource::FileSource: ERROR: Unsupported scheme in URL \"" << m_url.toString() << "\"" << std::endl;
+        cerr << "FileSource::FileSource: ERROR: Unsupported scheme in URL \"" << m_url.toString() << "\"" << endl;
         m_errorString = tr("Unsupported scheme in URL");
         return;
     }
@@ -184,7 +208,7 @@ FileSource::FileSource(QUrl url, ProgressReporter *reporter) :
     init();
 
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource::FileSource(url) exiting" << std::endl;
+    cerr << "FileSource::FileSource(url) exiting" << endl;
 #endif
 }
 
@@ -194,6 +218,7 @@ FileSource::FileSource(const FileSource &rf) :
     m_localFile(0),
     m_reply(0),
     m_ok(rf.m_ok),
+    m_cancelled(rf.m_cancelled),
     m_lastStatus(rf.m_lastStatus),
     m_resource(rf.m_resource),
     m_remote(rf.m_remote),
@@ -203,12 +228,12 @@ FileSource::FileSource(const FileSource &rf) :
     m_refCounted(false)
 {
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource::FileSource(" << m_url.toString() << ") [copy ctor]" << std::endl;
+    cerr << "FileSource::FileSource(" << m_url.toString() << ") [copy ctor]" << endl;
     incCount(m_url.toString());
 #endif
 
     if (!canHandleScheme(m_url)) {
-        std::cerr << "FileSource::FileSource: ERROR: Unsupported scheme in URL \"" << m_url.toString() << "\"" << std::endl;
+        cerr << "FileSource::FileSource: ERROR: Unsupported scheme in URL \"" << m_url.toString() << "\"" << endl;
         m_errorString = tr("Unsupported scheme in URL");
         return;
     }
@@ -218,13 +243,13 @@ FileSource::FileSource(const FileSource &rf) :
     } else {
         QMutexLocker locker(&m_mapMutex);
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::FileSource(copy ctor): ref count is "
-                  << m_refCountMap[m_url] << std::endl;
+        cerr << "FileSource::FileSource(copy ctor): ref count is "
+                  << m_refCountMap[m_url] << endl;
 #endif
         if (m_refCountMap[m_url] > 0) {
             m_refCountMap[m_url]++;
 #ifdef DEBUG_FILE_SOURCE
-            std::cerr << "raised it to " << m_refCountMap[m_url] << std::endl;
+            cerr << "raised it to " << m_refCountMap[m_url] << endl;
 #endif
             m_localFilename = m_remoteLocalMap[m_url];
             m_refCounted = true;
@@ -237,18 +262,18 @@ FileSource::FileSource(const FileSource &rf) :
     m_done = true;
 
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource::FileSource(" << m_url.toString() << ") [copy ctor]: note: local filename is \"" << m_localFilename << "\"" << std::endl;
+    cerr << "FileSource::FileSource(" << m_url.toString() << ") [copy ctor]: note: local filename is \"" << m_localFilename << "\"" << endl;
 #endif
 
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource::FileSource(copy ctor) exiting" << std::endl;
+    cerr << "FileSource::FileSource(copy ctor) exiting" << endl;
 #endif
 }
 
 FileSource::~FileSource()
 {
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource(" << m_url.toString() << ")::~FileSource" << std::endl;
+    cerr << "FileSource(" << m_url.toString() << ")::~FileSource" << endl;
     decCount(m_url.toString());
 #endif
 
@@ -262,14 +287,14 @@ FileSource::init()
 {
     if (isResource()) {
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::init: Is a resource" << std::endl;
+        cerr << "FileSource::init: Is a resource" << endl;
 #endif
         QString resourceFile = m_url.toString();
         resourceFile.replace(QRegExp("^qrc:"), ":");
         
         if (!QFileInfo(resourceFile).exists()) {
 #ifdef DEBUG_FILE_SOURCE
-            std::cerr << "FileSource::init: Resource file of this name does not exist, switching to non-resource URL" << std::endl;
+            cerr << "FileSource::init: Resource file of this name does not exist, switching to non-resource URL" << endl;
 #endif
             m_url = resourceFile;
             m_resource = false;
@@ -278,21 +303,26 @@ FileSource::init()
 
     if (!isRemote() && !isResource()) {
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::init: Not a remote URL" << std::endl;
+        cerr << "FileSource::init: Not a remote URL" << endl;
 #endif
         bool literal = false;
         m_localFilename = m_url.toLocalFile();
+
         if (m_localFilename == "") {
             // QUrl may have mishandled the scheme (e.g. in a DOS path)
-            m_localFilename = m_url.toString();
+            m_localFilename = m_rawFileOrUrl;
+#ifdef DEBUG_FILE_SOURCE
+            cerr << "FileSource::init: Trying literal local filename \""
+                      << m_localFilename << "\"" << endl;
+#endif
             literal = true;
         }
         m_localFilename = QFileInfo(m_localFilename).absoluteFilePath();
 
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::init: URL translates to local filename \""
+        cerr << "FileSource::init: URL translates to absolute filename \""
                   << m_localFilename << "\" (with literal=" << literal << ")"
-                  << std::endl;
+                  << endl;
 #endif
         m_ok = true;
         m_lastStatus = 200;
@@ -302,11 +332,11 @@ FileSource::init()
                 m_lastStatus = 404;
             } else {
 #ifdef DEBUG_FILE_SOURCE
-                std::cerr << "FileSource::init: Local file of this name does not exist, trying URL as a literal filename" << std::endl;
+                cerr << "FileSource::init: Local file of this name does not exist, trying URL as a literal filename" << endl;
 #endif
                 // Again, QUrl may have been mistreating us --
                 // e.g. dropping a part that looks like query data
-                m_localFilename = m_url.toString();
+                m_localFilename = m_rawFileOrUrl;
                 literal = true;
                 if (!QFileInfo(m_localFilename).exists()) {
                     m_lastStatus = 404;
@@ -320,7 +350,7 @@ FileSource::init()
 
     if (createCacheFile()) {
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::init: Already have this one" << std::endl;
+        cerr << "FileSource::init: Already have this one" << endl;
 #endif
         m_ok = true;
         if (!QFileInfo(m_localFilename).exists()) {
@@ -349,7 +379,7 @@ FileSource::init()
         QByteArray ba(resourceFile.readAll());
         
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "Copying " << ba.size() << " bytes from resource file to cache file" << std::endl;
+        cerr << "Copying " << ba.size() << " bytes from resource file to cache file" << endl;
 #endif
 
         qint64 written = m_localFile->write(ba);
@@ -359,7 +389,7 @@ FileSource::init()
 
         if (written != ba.size()) {
 #ifdef DEBUG_FILE_SOURCE
-            std::cerr << "Copy failed (wrote " << written << " bytes)" << std::endl;
+            cerr << "Copy failed (wrote " << written << " bytes)" << endl;
 #endif
             m_ok = false;
             return;
@@ -374,14 +404,14 @@ FileSource::init()
         QString scheme = m_url.scheme().toLower();
 
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::init: Don't have local copy of \""
-                  << m_url.toString() << "\", retrieving" << std::endl;
+        cerr << "FileSource::init: Don't have local copy of \""
+                  << m_url.toString() << "\", retrieving" << endl;
 #endif
 
         if (scheme == "http" || scheme == "https" || scheme == "ftp") {
             initRemote();
 #ifdef DEBUG_FILE_SOURCE
-            std::cerr << "FileSource: initRemote returned" << std::endl;
+            cerr << "FileSource: initRemote returned" << endl;
 #endif
         } else {
             m_remote = false;
@@ -399,7 +429,7 @@ FileSource::init()
             cleanup();
             m_refCountMap[m_url]++;
 #ifdef DEBUG_FILE_SOURCE
-            std::cerr << "FileSource::init: Another FileSource has got there first, abandoning our download and using theirs" << std::endl;
+            cerr << "FileSource::init: Another FileSource has got there first, abandoning our download and using theirs" << endl;
 #endif
             m_localFilename = m_remoteLocalMap[m_url];
             m_refCounted = true;
@@ -435,15 +465,25 @@ FileSource::initRemote()
     
     if (m_preferredContentType != "") {
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource: indicating preferred content type of \""
-                  << m_preferredContentType << "\"" << std::endl;
+        cerr << "FileSource: indicating preferred content type of \""
+                  << m_preferredContentType << "\"" << endl;
 #endif
         req.setRawHeader
             ("Accept",
              QString("%1, */*").arg(m_preferredContentType).toLatin1());
     }
 
-    m_reply = nm.get(req);
+    { // check we have a QNetworkAccessManager
+        QMutexLocker locker(&m_mapMutex);
+        if (!nms.hasLocalData()) {
+#ifdef DEBUG_FILE_SOURCE
+            ++threadCount;
+#endif
+            nms.setLocalData(new QNetworkAccessManager());
+        }
+    }
+
+    m_reply = nms.localData()->get(req);
 
     connect(m_reply, SIGNAL(readyRead()),
             this, SLOT(readyRead()));
@@ -467,8 +507,12 @@ FileSource::cleanup()
     m_done = true;
     if (m_reply) {
         QNetworkReply *r = m_reply;
+        disconnect(r, 0, this, 0);
         m_reply = 0;
-        r->abort();
+        // Can only call abort() when there are no errors.
+        if (r->error() == QNetworkReply::NoError) {
+            r->abort();
+        }
         r->deleteLater();
     }
     if (m_localFile) {
@@ -501,11 +545,14 @@ FileSource::isAvailable()
 {
     waitForStatus();
     bool available = true;
-    if (!m_ok) available = false;
-    else available = (m_lastStatus / 100 == 2);
+    if (!m_ok) {
+        available = false;
+    } else {
+        // http 2xx status codes mean success
+        available = (m_lastStatus / 100 == 2);
+    }
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource::isAvailable: " << (available ? "yes" : "no")
-              << std::endl;
+    cerr << "FileSource::isAvailable: " << (available ? "yes" : "no") << endl;
 #endif
     return available;
 }
@@ -514,7 +561,7 @@ void
 FileSource::waitForStatus()
 {
     while (m_ok && (!m_done && m_lastStatus == 0)) {
-//        std::cerr << "waitForStatus: processing (last status " << m_lastStatus << ")" << std::endl;
+//        cerr << "waitForStatus: processing (last status " << m_lastStatus << ")" << endl;
         QCoreApplication::processEvents();
     }
 }
@@ -523,7 +570,7 @@ void
 FileSource::waitForData()
 {
     while (m_ok && !m_done) {
-//        std::cerr << "FileSource::waitForData: calling QApplication::processEvents" << std::endl;
+//        cerr << "FileSource::waitForData: calling QApplication::processEvents" << endl;
         QCoreApplication::processEvents();
         usleep(10000);
     }
@@ -547,6 +594,12 @@ FileSource::isDone() const
     return m_done;
 }
 
+bool
+FileSource::wasCancelled() const
+{
+    return m_cancelled;
+}
+
 bool
 FileSource::isResource() const
 {
@@ -609,23 +662,26 @@ void
 FileSource::metaDataChanged()
 {
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource::metaDataChanged" << std::endl;
+    cerr << "FileSource::metaDataChanged" << endl;
 #endif
 
     if (!m_reply) {
-        std::cerr << "WARNING: FileSource::metaDataChanged() called without a reply object being known to us" << std::endl;
+        cerr << "WARNING: FileSource::metaDataChanged() called without a reply object being known to us" << endl;
         return;
     }
 
+    // Handle http transfer status codes.
+
     int status =
         m_reply->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
 
+    // If this is a redirection (3xx) code, do the redirect
     if (status / 100 == 3) {
         QString location = m_reply->header
             (QNetworkRequest::LocationHeader).toString();
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::metaDataChanged: redirect to \""
-                  << location << "\" received" << std::endl;
+        cerr << "FileSource::metaDataChanged: redirect to \""
+                  << location << "\" received" << endl;
 #endif
         if (location != "") {
             QUrl newUrl(location);
@@ -648,19 +704,21 @@ FileSource::metaDataChanged()
     }
 
     m_lastStatus = status;
+
+    // 400 and up are failures, get the error string
     if (m_lastStatus / 100 >= 4) {
         m_errorString = QString("%1 %2")
             .arg(status)
             .arg(m_reply->attribute
                  (QNetworkRequest::HttpReasonPhraseAttribute).toString());
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::metaDataChanged: "
-                  << m_errorString << std::endl;
+        cerr << "FileSource::metaDataChanged: "
+                  << m_errorString << endl;
 #endif
     } else {
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::metaDataChanged: "
-                  << m_lastStatus << std::endl;
+        cerr << "FileSource::metaDataChanged: "
+                  << m_lastStatus << endl;
 #endif
         m_contentType =
             m_reply->header(QNetworkRequest::ContentTypeHeader).toString();
@@ -682,6 +740,7 @@ FileSource::cancelled()
     cleanup();
 
     m_ok = false;
+    m_cancelled = true;
     m_errorString = tr("Download cancelled");
 }
 
@@ -691,11 +750,18 @@ FileSource::replyFinished()
     emit progress(100);
 
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource::replyFinished()" << std::endl;
+    cerr << "FileSource::replyFinished()" << endl;
 #endif
 
     if (m_done) return;
 
+    QString scheme = m_url.scheme().toLower();
+    // For ftp transfers, replyFinished() will be called on success.
+    // metaDataChanged() is never called for ftp transfers.
+    if (scheme == "ftp") {
+        m_lastStatus = 200;  // http ok
+    }
+
     bool error = (m_lastStatus / 100 >= 4);
 
     cleanup();
@@ -713,7 +779,7 @@ FileSource::replyFinished()
 
     if (error) {
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::done: error is " << error << ", deleting cache file" << std::endl;
+        cerr << "FileSource::done: error is " << error << ", deleting cache file" << endl;
 #endif
         deleteCacheFile();
     }
@@ -728,7 +794,11 @@ void
 FileSource::replyFailed(QNetworkReply::NetworkError)
 {
     emit progress(100);
-    m_errorString = m_reply->errorString();
+    if (!m_reply) {
+        cerr << "WARNING: FileSource::replyFailed() called without a reply object being known to us" << endl;
+    } else {
+        m_errorString = m_reply->errorString();
+    }
     m_ok = false;
     m_done = true;
     cleanup();
@@ -739,7 +809,7 @@ void
 FileSource::deleteCacheFile()
 {
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource::deleteCacheFile(\"" << m_localFilename << "\")" << std::endl;
+    cerr << "FileSource::deleteCacheFile(\"" << m_localFilename << "\")" << endl;
 #endif
 
     cleanup();
@@ -750,7 +820,7 @@ FileSource::deleteCacheFile()
 
     if (!isRemote()) {
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "not a cache file" << std::endl;
+        cerr << "not a cache file" << endl;
 #endif
         return;
     }
@@ -763,7 +833,7 @@ FileSource::deleteCacheFile()
         if (m_refCountMap[m_url] > 0) {
             m_refCountMap[m_url]--;
 #ifdef DEBUG_FILE_SOURCE
-            std::cerr << "reduced ref count to " << m_refCountMap[m_url] << std::endl;
+            cerr << "reduced ref count to " << m_refCountMap[m_url] << endl;
 #endif
             if (m_refCountMap[m_url] > 0) {
                 m_done = true;
@@ -776,11 +846,11 @@ FileSource::deleteCacheFile()
 
     if (!QFile(m_localFilename).remove()) {
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::deleteCacheFile: ERROR: Failed to delete file \"" << m_localFilename << "\"" << std::endl;
+        cerr << "FileSource::deleteCacheFile: ERROR: Failed to delete file \"" << m_localFilename << "\"" << endl;
 #endif
     } else {
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::deleteCacheFile: Deleted cache file \"" << m_localFilename << "\"" << std::endl;
+        cerr << "FileSource::deleteCacheFile: Deleted cache file \"" << m_localFilename << "\"" << endl;
 #endif
         m_localFilename = "";
     }
@@ -797,14 +867,14 @@ FileSource::createCacheFile()
         QMutexLocker locker(&m_mapMutex);
 
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::createCacheFile: refcount is " << m_refCountMap[m_url] << std::endl;
+        cerr << "FileSource::createCacheFile: refcount is " << m_refCountMap[m_url] << endl;
 #endif
 
         if (m_refCountMap[m_url] > 0) {
             m_refCountMap[m_url]++;
             m_localFilename = m_remoteLocalMap[m_url];
 #ifdef DEBUG_FILE_SOURCE
-            std::cerr << "raised it to " << m_refCountMap[m_url] << std::endl;
+            cerr << "raised it to " << m_refCountMap[m_url] << endl;
 #endif
             m_refCounted = true;
             return true;
@@ -816,9 +886,9 @@ FileSource::createCacheFile()
         dir = TempDirectory::getInstance()->getSubDirectoryPath("download");
     } catch (DirectoryCreationFailed f) {
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::createCacheFile: ERROR: Failed to create temporary directory: " << f.what() << std::endl;
+        cerr << "FileSource::createCacheFile: ERROR: Failed to create temporary directory: " << f.what() << endl;
 #endif
-        return "";
+        return false;
     }
 
     QString filepart = m_url.path().section('/', -1, -1,
@@ -844,7 +914,7 @@ FileSource::createCacheFile()
     QString filepath(dir.filePath(filename));
 
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource::createCacheFile: URL is \"" << m_url.toString() << "\", dir is \"" << dir.path() << "\", base \"" << base << "\", extension \"" << extension << "\", filebase \"" << filename << "\", filename \"" << filepath << "\"" << std::endl;
+    cerr << "FileSource::createCacheFile: URL is \"" << m_url.toString() << "\", dir is \"" << dir.path() << "\", base \"" << base << "\", extension \"" << extension << "\", filebase \"" << filename << "\", filename \"" << filepath << "\"" << endl;
 #endif
 
     QMutexLocker fcLocker(&m_fileCreationMutex);
@@ -855,9 +925,9 @@ FileSource::createCacheFile()
         !QFile(filepath).open(QFile::WriteOnly)) {
 
 #ifdef DEBUG_FILE_SOURCE
-        std::cerr << "FileSource::createCacheFile: Failed to create local file \""
+        cerr << "FileSource::createCacheFile: Failed to create local file \""
                   << filepath << "\" for URL \""
-                  << m_url.toString() << "\" (or file already exists): appending suffix instead" << std::endl;
+                  << m_url.toString() << "\" (or file already exists): appending suffix instead" << endl;
 #endif
 
         if (extension == "") {
@@ -871,19 +941,19 @@ FileSource::createCacheFile()
             !QFile(filepath).open(QFile::WriteOnly)) {
 
 #ifdef DEBUG_FILE_SOURCE
-            std::cerr << "FileSource::createCacheFile: ERROR: Failed to create local file \""
+            cerr << "FileSource::createCacheFile: ERROR: Failed to create local file \""
                       << filepath << "\" for URL \""
-                      << m_url.toString() << "\" (or file already exists)" << std::endl;
+                      << m_url.toString() << "\" (or file already exists)" << endl;
 #endif
 
-            return "";
+            return false;
         }
     }
 
 #ifdef DEBUG_FILE_SOURCE
-    std::cerr << "FileSource::createCacheFile: url "
+    cerr << "FileSource::createCacheFile: url "
               << m_url.toString() << " -> local filename "
-              << filepath << std::endl;
+              << filepath << endl;
 #endif
     
     m_localFilename = filepath;
