@@ -36,14 +36,15 @@
 #include "model/NoteModel.h"
 
 #include <QString>
+#include <QFileInfo>
 
 #include <sstream>
 
+#include "base/Debug.h"
+
 using std::string;
 using std::ifstream;
 using std::stringstream;
-using std::cerr;
-using std::endl;
 using std::ends;
 using std::ios;
 using std::vector;
@@ -52,12 +53,12 @@ using std::set;
 
 using namespace MIDIConstants;
 
-//#define MIDI_SVDEBUG 1
+//#define MIDI_DEBUG 1
 
 
 MIDIFileReader::MIDIFileReader(QString path,
                                MIDIFileImportPreferenceAcquirer *acquirer,
-			       size_t mainModelSampleRate) :
+			       sv_samplerate_t mainModelSampleRate) :
     m_smpte(false),
     m_timingDivision(0),
     m_fps(0),
@@ -300,7 +301,9 @@ MIDIFileReader::parseFile()
 	// Set file size so we can count it off
 	//
 	m_midiFile->seekg(0, ios::end);
-	m_fileSize = m_midiFile->tellg();
+        std::streamoff off = m_midiFile->tellg();
+	m_fileSize = 0;
+        if (off > 0) m_fileSize = off;
 	m_midiFile->seekg(0, ios::beg);
 
 	// Parse the MIDI header first.  The first 14 bytes of the file.
@@ -320,7 +323,7 @@ MIDIFileReader::parseFile()
 
 	    if (!skipToNextTrack()) {
 #ifdef MIDI_DEBUG
-		cerr << "Couldn't find Track " << j << endl;
+		SVDEBUG << "Couldn't find Track " << j << endl;
 #endif
 		m_error = "File corrupted or in non-standard format?";
 		m_format = MIDI_FILE_BAD_FORMAT;
@@ -328,14 +331,14 @@ MIDIFileReader::parseFile()
 	    }
 
 #ifdef MIDI_DEBUG
-	    cerr << "Track has " << m_trackByteCount << " bytes" << endl;
+	    SVDEBUG << "Track has " << m_trackByteCount << " bytes" << endl;
 #endif
 
 	    // Run through the events taking them into our internal
 	    // representation.
 	    if (!parseTrack(i)) {
 #ifdef MIDI_DEBUG
-		cerr << "Track " << j << " parsing failed" << endl;
+		SVDEBUG << "Track " << j << " parsing failed" << endl;
 #endif
 		m_error = "File corrupted or in non-standard format?";
 		m_format = MIDI_FILE_BAD_FORMAT;
@@ -439,9 +442,9 @@ MIDIFileReader::parseTrack(unsigned int &lastTrackNum)
     MIDIByte midiByte, metaEventCode, data1, data2;
     MIDIByte eventCode = 0x80;
     string metaMessage;
-    unsigned int messageLength;
-    unsigned long deltaTime;
-    unsigned long accumulatedTime = 0;
+    long messageLength;
+    long deltaTime;
+    long accumulatedTime = 0;
 
     // The trackNum passed in to this method is the default track for
     // all events provided they're all on the same channel.  If we find
@@ -473,7 +476,7 @@ MIDIFileReader::parseTrack(unsigned int &lastTrackNum)
 
 	if (eventCode < 0x80) {
 #ifdef MIDI_DEBUG
-	    cerr << "WARNING: Invalid event code " << eventCode
+	    SVDEBUG << "WARNING: Invalid event code " << eventCode
 		 << " in MIDI file" << endl;
 #endif
 	    throw MIDIException(tr("Invalid event code %1 found").arg(int(eventCode)));
@@ -482,7 +485,7 @@ MIDIFileReader::parseTrack(unsigned int &lastTrackNum)
         deltaTime = getNumberFromMIDIBytes();
 
 #ifdef MIDI_DEBUG
-	cerr << "read delta time " << deltaTime << endl;
+	SVDEBUG << "read delta time " << deltaTime << endl;
 #endif
 
         // Get a single byte
@@ -502,7 +505,7 @@ MIDIFileReader::parseTrack(unsigned int &lastTrackNum)
 #endif
         } else {
 #ifdef MIDI_DEBUG
-	    cerr << "have new event code " << int(midiByte) << endl;
+	    SVDEBUG << "have new event code " << int(midiByte) << endl;
 #endif
             eventCode = midiByte;
 	    data1 = getMIDIByte();
@@ -514,7 +517,7 @@ MIDIFileReader::parseTrack(unsigned int &lastTrackNum)
             messageLength = getNumberFromMIDIBytes();
 
 //#ifdef MIDI_DEBUG
-		cerr << "Meta event of type " << int(metaEventCode) << " and " << messageLength << " bytes found, putting on track " << metaTrack << endl;
+		SVDEBUG << "Meta event of type " << int(metaEventCode) << " and " << messageLength << " bytes found, putting on track " << metaTrack << endl;
 //#endif
             metaMessage = getMIDIBytes(messageLength);
 
@@ -569,7 +572,7 @@ MIDIFileReader::parseTrack(unsigned int &lastTrackNum)
                 midiEvent = new MIDIEvent(deltaTime, eventCode, data1, data2);
 
                 /*
-		cerr << "MIDI event for channel " << channel << " (track "
+		SVDEBUG << "MIDI event for channel " << channel << " (track "
 			  << trackNum << ")" << endl;
 		midiEvent->print();
                           */
@@ -602,7 +605,7 @@ MIDIFileReader::parseTrack(unsigned int &lastTrackNum)
                 messageLength = getNumberFromMIDIBytes(data1);
 
 #ifdef MIDI_DEBUG
-		cerr << "SysEx of " << messageLength << " bytes found" << endl;
+		SVDEBUG << "SysEx of " << messageLength << " bytes found" << endl;
 #endif
 
                 metaMessage= getMIDIBytes(messageLength);
@@ -629,13 +632,6 @@ MIDIFileReader::parseTrack(unsigned int &lastTrackNum)
                 m_midiComposition[trackNum].push_back(midiEvent);
                 break;
 
-            case MIDI_END_OF_EXCLUSIVE:
-#ifdef MIDI_DEBUG
-                SVDEBUG << "MIDIFileReader::parseTrack() - "
-                          << "Found a stray MIDI_END_OF_EXCLUSIVE" << endl;
-#endif
-                break;
-
             default:
 #ifdef MIDI_DEBUG
                 SVDEBUG << "MIDIFileReader::parseTrack()" 
@@ -713,7 +709,7 @@ MIDIFileReader::consolidateNoteOffEvents(unsigned int track)
 void
 MIDIFileReader::updateTempoMap(unsigned int track)
 {
-    std::cerr << "updateTempoMap for track " << track << " (" << m_midiComposition[track].size() << " events)" << std::endl;
+    SVDEBUG << "updateTempoMap for track " << track << " (" << m_midiComposition[track].size() << " events)" << endl;
 
     for (MIDITrack::iterator i = m_midiComposition[track].begin();
 	 i != m_midiComposition[track].end(); ++i) {
@@ -727,7 +723,7 @@ MIDIFileReader::updateTempoMap(unsigned int track)
 	    
 	    long tempo = (((m0 << 8) + m1) << 8) + m2;
 
-	    std::cerr << "updateTempoMap: have tempo, it's " << tempo << " at " << (*i)->getTime() << std::endl;
+	    SVDEBUG << "updateTempoMap: have tempo, it's " << tempo << " at " << (*i)->getTime() << endl;
 
 	    if (tempo != 0) {
 		double qpm = 60000000.0 / double(tempo);
@@ -790,11 +786,11 @@ MIDIFileReader::getTimeForMIDITime(unsigned long midiTime) const
     SVDEBUG << "MIDIFileReader::getTimeForMIDITime(" << midiTime << ")"
 	      << endl;
     SVDEBUG << "timing division = " << td << endl;
-    std::cerr << "nearest tempo event (of " << m_tempoMap.size() << ") is at " << tempoMIDITime << " ("
-	      << tempoRealTime << ")" << std::endl;
-    std::cerr << "quarters since then = " << quarters << std::endl;
-    std::cerr << "tempo = " << tempo << " quarters per minute" << std::endl;
-    std::cerr << "seconds since then = " << seconds << std::endl;
+    SVDEBUG << "nearest tempo event (of " << m_tempoMap.size() << ") is at " << tempoMIDITime << " ("
+	      << tempoRealTime << ")" << endl;
+    SVDEBUG << "quarters since then = " << quarters << endl;
+    SVDEBUG << "tempo = " << tempo << " quarters per minute" << endl;
+    SVDEBUG << "seconds since then = " << seconds << endl;
     SVDEBUG << "resulting time = " << (tempoRealTime + RealTime::fromSeconds(seconds)) << endl;
 */
 
@@ -896,7 +892,7 @@ MIDIFileReader::load() const
 
     if (tracksToLoad.empty()) return 0;
 
-    size_t n = tracksToLoad.size(), count = 0;
+    int n = int(tracksToLoad.size()), count = 0;
     Model *model = 0;
 
     for (std::set<unsigned int>::iterator i = tracksToLoad.begin();
@@ -932,21 +928,21 @@ MIDIFileReader::loadTrack(unsigned int trackToLoad,
     if (existingModel) {
 	model = dynamic_cast<NoteModel *>(existingModel);
 	if (!model) {
-	    std::cerr << "WARNING: MIDIFileReader::loadTrack: Existing model given, but it isn't a NoteModel -- ignoring it" << std::endl;
+	    SVDEBUG << "WARNING: MIDIFileReader::loadTrack: Existing model given, but it isn't a NoteModel -- ignoring it" << endl;
 	}
     }
 
     if (!model) {
 	model = new NoteModel(m_mainModelSampleRate, 1, 0.0, 0.0, false);
 	model->setValueQuantization(1.0);
+        model->setObjectName(QFileInfo(m_path).fileName());
     }
 
     const MIDITrack &track = m_midiComposition.find(trackToLoad)->second;
 
-    size_t totalEvents = track.size();
-    size_t count = 0;
+    int totalEvents = int(track.size());
+    int count = 0;
 
-    bool minorKey = false;
     bool sharpKey = true;
 
     for (MIDITrack::const_iterator i = track.begin(); i != track.end(); ++i) {
@@ -968,7 +964,7 @@ MIDIFileReader::loadTrack(unsigned int trackToLoad,
 	    switch((*i)->getMetaEventCode()) {
 
 	    case MIDI_KEY_SIGNATURE:
-		minorKey = (int((*i)->getMetaMessage()[1]) != 0);
+		// minorKey = (int((*i)->getMetaMessage()[1]) != 0);
 		sharpKey = (int((*i)->getMetaMessage()[0]) >= 0);
 		break;
 
