@@ -30,7 +30,7 @@
 #include "data/model/NoteModel.h"
 #include "data/model/TextModel.h"
 #include "data/model/RegionModel.h"
-#include "data/model/WaveFileModel.h"
+#include "data/model/ReadOnlyWaveFileModel.h"
 
 #include "data/fileio/FileSource.h"
 #include "data/fileio/CachedFile.h"
@@ -47,16 +47,13 @@ using Dataquay::Triples;
 using Dataquay::BasicStore;
 using Dataquay::PropertyObject;
 
-using std::cerr;
-using std::endl;
-
 class RDFImporterImpl
 {
 public:
-    RDFImporterImpl(QString url, int sampleRate);
+    RDFImporterImpl(QString url, sv_samplerate_t sampleRate);
     virtual ~RDFImporterImpl();
 
-    void setSampleRate(int sampleRate) { m_sampleRate = sampleRate; }
+    void setSampleRate(sv_samplerate_t sampleRate) { m_sampleRate = sampleRate; }
     
     bool isOK();
     QString getErrorString() const;
@@ -70,7 +67,7 @@ protected:
     QString m_uristring;
     QString m_errorString;
     std::map<QString, Model *> m_audioModelMap;
-    int m_sampleRate;
+    sv_samplerate_t m_sampleRate;
 
     std::map<Model *, std::map<QString, float> > m_labelValueMap;
 
@@ -81,10 +78,11 @@ protected:
     void getDenseModelTitle(Model *, QString, QString);
 
     void getDenseFeatureProperties(QString featureUri,
-                                   int &sampleRate, int &windowLength,
+                                   sv_samplerate_t &sampleRate, int &windowLength,
                                    int &hopSize, int &width, int &height);
 
-    void fillModel(Model *, long, long, bool, std::vector<float> &, QString);
+    void fillModel(Model *, sv_frame_t, sv_frame_t,
+                   bool, std::vector<float> &, QString);
 };
 
 QString
@@ -93,7 +91,7 @@ RDFImporter::getKnownExtensions()
     return "*.rdf *.n3 *.ttl";
 }
 
-RDFImporter::RDFImporter(QString url, int sampleRate) :
+RDFImporter::RDFImporter(QString url, sv_samplerate_t sampleRate) :
     m_d(new RDFImporterImpl(url, sampleRate)) 
 {
 }
@@ -104,7 +102,7 @@ RDFImporter::~RDFImporter()
 }
 
 void
-RDFImporter::setSampleRate(int sampleRate)
+RDFImporter::setSampleRate(sv_samplerate_t sampleRate)
 {
     m_d->setSampleRate(sampleRate);
 }
@@ -127,7 +125,7 @@ RDFImporter::getDataModels(ProgressReporter *r)
     return m_d->getDataModels(r);
 }
 
-RDFImporterImpl::RDFImporterImpl(QString uri, int sampleRate) :
+RDFImporterImpl::RDFImporterImpl(QString uri, sv_samplerate_t sampleRate) :
     m_store(new BasicStore),
     m_uristring(uri),
     m_sampleRate(sampleRate)
@@ -180,7 +178,7 @@ RDFImporterImpl::getDataModels(ProgressReporter *reporter)
 
     if (m_sampleRate == 0) {
         m_errorString = QString("Invalid audio data model (is audio file format supported?)");
-        std::cerr << m_errorString << std::endl;
+        cerr << m_errorString << endl;
         return models;
     }
 
@@ -221,7 +219,7 @@ RDFImporterImpl::getDataModelsAudio(std::vector<Model *> &models,
             file = m_store->complete(Triple(sig, expand("mo:available_as"), Node()));
         }
         if (file == Node()) {
-            std::cerr << "RDFImporterImpl::getDataModelsAudio: ERROR: No source for signal " << sig << std::endl;
+            cerr << "RDFImporterImpl::getDataModelsAudio: ERROR: No source for signal " << sig << endl;
             continue;
         }
 
@@ -254,8 +252,8 @@ RDFImporterImpl::getDataModelsAudio(std::vector<Model *> &models,
                                         fs->getLocation(),
                                         m_uristring);
                 if (path != "") {
-                    std::cerr << "File finder returns: \"" << path.toStdString()
-                              << "\"" << std::endl;
+                    cerr << "File finder returns: \"" << path
+                              << "\"" << endl;
                     delete fs;
                     fs = new FileSource(path, reporter);
                     if (!fs->isAvailable()) {
@@ -272,9 +270,9 @@ RDFImporterImpl::getDataModelsAudio(std::vector<Model *> &models,
             reporter->setMessage(RDFImporter::tr("Importing audio referenced in RDF..."));
         }
         fs->waitForData();
-        WaveFileModel *newModel = new WaveFileModel(*fs, m_sampleRate);
+        ReadOnlyWaveFileModel *newModel = new ReadOnlyWaveFileModel(*fs, m_sampleRate);
         if (newModel->isOK()) {
-            std::cerr << "Successfully created wave file model from source at \"" << source << "\"" << std::endl;
+            cerr << "Successfully created wave file model from source at \"" << source << "\"" << endl;
             models.push_back(newModel);
             m_audioModelMap[signal] = newModel;
             if (m_sampleRate == 0) {
@@ -312,7 +310,7 @@ RDFImporterImpl::getDataModelsDense(std::vector<Model *> &models,
         
         if (type == "" || value == "") continue;
 
-        int sampleRate = 0;
+        sv_samplerate_t sampleRate = 0;
         int windowLength = 0;
         int hopSize = 0;
         int width = 0;
@@ -419,7 +417,7 @@ RDFImporterImpl::getDenseModelTitle(Model *m,
 
 void
 RDFImporterImpl::getDenseFeatureProperties(QString featureUri,
-                                           int &sampleRate, int &windowLength,
+                                           sv_samplerate_t &sampleRate, int &windowLength,
                                            int &hopSize, int &width, int &height)
 {
     Node dim = m_store->complete
@@ -470,7 +468,7 @@ RDFImporterImpl::getDenseFeatureProperties(QString featureUri,
     PropertyObject po(m_store, "tl:", map);
 
     if (po.hasProperty("sampleRate")) {
-        sampleRate = po.getProperty("sampleRate").toInt();
+        sampleRate = po.getProperty("sampleRate").toDouble();
     }
     if (po.hasProperty("hopSize")) {
         hopSize = po.getProperty("hopSize").toInt();
@@ -579,14 +577,14 @@ RDFImporterImpl::getDataModelsSparse(std::vector<Model *> &models,
                 RealTime time;
                 RealTime duration;
 
-                bool haveTime = false;
+//                bool haveTime = false;
                 bool haveDuration = false;
 
                 Node at = m_store->complete(Triple(tn, expand("tl:at"), Node()));
 
                 if (at != Node()) {
                     time = RealTime::fromXsdDuration(at.value.toStdString());
-                    haveTime = true;
+//                    haveTime = true;
                 } else {
     //!!! NB we're using rather old terminology for these things, apparently:
     // beginsAt -> start
@@ -599,7 +597,7 @@ RDFImporterImpl::getDataModelsSparse(std::vector<Model *> &models,
                             (start.value.toStdString());
                         duration = RealTime::fromXsdDuration
                             (dur.value.toStdString());
-                        haveTime = haveDuration = true;
+//                        haveTime = haveDuration = true;
                     }
                 }
 
@@ -669,7 +667,7 @@ RDFImporterImpl::getDataModelsSparse(std::vector<Model *> &models,
                     model->setRDFTypeURI(type);
 
                     if (m_audioModelMap.find(source) != m_audioModelMap.end()) {
-                        std::cerr << "source model for " << model << " is " << m_audioModelMap[source] << std::endl;
+                        cerr << "source model for " << model << " is " << m_audioModelMap[source] << endl;
                         model->setSourceModel(m_audioModelMap[source]);
                     }
 
@@ -689,8 +687,8 @@ RDFImporterImpl::getDataModelsSparse(std::vector<Model *> &models,
                 model = modelMap[timeline][type][dimensions][haveDuration];
 
                 if (model) {
-                    long ftime = RealTime::realTime2Frame(time, m_sampleRate);
-                    long fduration = RealTime::realTime2Frame(duration, m_sampleRate);
+                    sv_frame_t ftime = RealTime::realTime2Frame(time, m_sampleRate);
+                    sv_frame_t fduration = RealTime::realTime2Frame(duration, m_sampleRate);
                     fillModel(model, ftime, fduration, haveDuration, values, label);
                 }
             }
@@ -700,8 +698,8 @@ RDFImporterImpl::getDataModelsSparse(std::vector<Model *> &models,
 
 void
 RDFImporterImpl::fillModel(Model *model,
-                           long ftime,
-                           long fduration,
+                           sv_frame_t ftime,
+                           sv_frame_t fduration,
                            bool haveDuration,
                            std::vector<float> &values,
                            QString label)
@@ -760,7 +758,8 @@ RDFImporterImpl::fillModel(Model *model,
                     }
                 }
             }
-            NoteModel::Point point(ftime, value, duration, level, label);
+            NoteModel::Point point(ftime, value, sv_frame_t(lrintf(duration)),
+                                   level, label);
             nm->addPoint(point);
         }
         return;
@@ -792,13 +791,14 @@ RDFImporterImpl::fillModel(Model *model,
                     duration = values[1];
                 }
             }
-            RegionModel::Point point(ftime, value, duration, label);
+            RegionModel::Point point(ftime, value,
+                                     sv_frame_t(lrintf(duration)), label);
             rm->addPoint(point);
         }
         return;
     }
             
-    std::cerr << "WARNING: RDFImporterImpl::fillModel: Unknown or unexpected model type" << std::endl;
+    cerr << "WARNING: RDFImporterImpl::fillModel: Unknown or unexpected model type" << endl;
     return;
 }
 
