@@ -15,6 +15,7 @@
 
 #include "ViewManager.h"
 #include "base/AudioPlaySource.h"
+#include "base/AudioRecordTarget.h"
 #include "base/RealTime.h"
 #include "data/model/Model.h"
 #include "widgets/CommandHistory.h"
@@ -30,6 +31,7 @@
 
 ViewManager::ViewManager() :
     m_playSource(0),
+    m_recordTarget(0),
     m_globalCentreFrame(0),
     m_globalZoom(1024),
     m_playbackFrame(0),
@@ -43,11 +45,12 @@ ViewManager::ViewManager() :
     m_playSelectionMode(false),
     m_playSoloMode(false),
     m_alignMode(false),
-    m_overlayMode(MinimalOverlays),
+    m_overlayMode(StandardOverlays),
     m_zoomWheelsEnabled(true),
     m_showCentreLine(true),
     m_illuminateLocalFeatures(true),
     m_showWorkTitle(false),
+    m_showDuration(true),
     m_lightPalette(QApplication::palette()),
     m_darkPalette(QApplication::palette())
 {
@@ -57,9 +60,9 @@ ViewManager::ViewManager() :
         (settings.value("overlay-mode", int(m_overlayMode)).toInt());
 
     if (m_overlayMode != NoOverlays &&
-        m_overlayMode != MinimalOverlays &&
+        m_overlayMode != StandardOverlays &&
         m_overlayMode != AllOverlays) {
-        m_overlayMode = MinimalOverlays;
+        m_overlayMode = StandardOverlays;
     }
 
     m_zoomWheelsEnabled =
@@ -70,18 +73,18 @@ ViewManager::ViewManager() :
 
     if (getGlobalDarkBackground()) {
 /*
-        std::cerr << "dark palette:" << std::endl;
-        std::cerr << "window = " << QApplication::palette().color(QPalette::Window).name() << std::endl;
-        std::cerr << "windowtext = " << QApplication::palette().color(QPalette::WindowText).name() << std::endl;
-        std::cerr << "base = " << QApplication::palette().color(QPalette::Base).name() << std::endl;
-        std::cerr << "alternatebase = " << QApplication::palette().color(QPalette::AlternateBase).name() << std::endl;
-        std::cerr << "text = " << QApplication::palette().color(QPalette::Text).name() << std::endl;
-        std::cerr << "button = " << QApplication::palette().color(QPalette::Button).name() << std::endl;
-        std::cerr << "buttontext = " << QApplication::palette().color(QPalette::ButtonText).name() << std::endl;
-        std::cerr << "brighttext = " << QApplication::palette().color(QPalette::BrightText).name() << std::endl;
-        std::cerr << "light = " << QApplication::palette().color(QPalette::Light).name() << std::endl;
-        std::cerr << "dark = " << QApplication::palette().color(QPalette::Dark).name() << std::endl;
-        std::cerr << "mid = " << QApplication::palette().color(QPalette::Mid).name() << std::endl;
+        cerr << "dark palette:" << endl;
+        cerr << "window = " << QApplication::palette().color(QPalette::Window).name() << endl;
+        cerr << "windowtext = " << QApplication::palette().color(QPalette::WindowText).name() << endl;
+        cerr << "base = " << QApplication::palette().color(QPalette::Base).name() << endl;
+        cerr << "alternatebase = " << QApplication::palette().color(QPalette::AlternateBase).name() << endl;
+        cerr << "text = " << QApplication::palette().color(QPalette::Text).name() << endl;
+        cerr << "button = " << QApplication::palette().color(QPalette::Button).name() << endl;
+        cerr << "buttontext = " << QApplication::palette().color(QPalette::ButtonText).name() << endl;
+        cerr << "brighttext = " << QApplication::palette().color(QPalette::BrightText).name() << endl;
+        cerr << "light = " << QApplication::palette().color(QPalette::Light).name() << endl;
+        cerr << "dark = " << QApplication::palette().color(QPalette::Dark).name() << endl;
+        cerr << "mid = " << QApplication::palette().color(QPalette::Mid).name() << endl;
 */
         m_lightPalette = QPalette(QColor("#000000"),  // WindowText
                                   QColor("#dddfe4"),  // Button
@@ -96,18 +99,18 @@ ViewManager::ViewManager() :
 
     } else {
 /*
-        std::cerr << "light palette:" << std::endl;
-        std::cerr << "window = " << QApplication::palette().color(QPalette::Window).name() << std::endl;
-        std::cerr << "windowtext = " << QApplication::palette().color(QPalette::WindowText).name() << std::endl;
-        std::cerr << "base = " << QApplication::palette().color(QPalette::Base).name() << std::endl;
-        std::cerr << "alternatebase = " << QApplication::palette().color(QPalette::AlternateBase).name() << std::endl;
-        std::cerr << "text = " << QApplication::palette().color(QPalette::Text).name() << std::endl;
-        std::cerr << "button = " << QApplication::palette().color(QPalette::Button).name() << std::endl;
-        std::cerr << "buttontext = " << QApplication::palette().color(QPalette::ButtonText).name() << std::endl;
-        std::cerr << "brighttext = " << QApplication::palette().color(QPalette::BrightText).name() << std::endl;
-        std::cerr << "light = " << QApplication::palette().color(QPalette::Light).name() << std::endl;
-        std::cerr << "dark = " << QApplication::palette().color(QPalette::Dark).name() << std::endl;
-        std::cerr << "mid = " << QApplication::palette().color(QPalette::Mid).name() << std::endl;
+        cerr << "light palette:" << endl;
+        cerr << "window = " << QApplication::palette().color(QPalette::Window).name() << endl;
+        cerr << "windowtext = " << QApplication::palette().color(QPalette::WindowText).name() << endl;
+        cerr << "base = " << QApplication::palette().color(QPalette::Base).name() << endl;
+        cerr << "alternatebase = " << QApplication::palette().color(QPalette::AlternateBase).name() << endl;
+        cerr << "text = " << QApplication::palette().color(QPalette::Text).name() << endl;
+        cerr << "button = " << QApplication::palette().color(QPalette::Button).name() << endl;
+        cerr << "buttontext = " << QApplication::palette().color(QPalette::ButtonText).name() << endl;
+        cerr << "brighttext = " << QApplication::palette().color(QPalette::BrightText).name() << endl;
+        cerr << "light = " << QApplication::palette().color(QPalette::Light).name() << endl;
+        cerr << "dark = " << QApplication::palette().color(QPalette::Dark).name() << endl;
+        cerr << "mid = " << QApplication::palette().color(QPalette::Mid).name() << endl;
 */
         m_darkPalette = QPalette(QColor("#ffffff"),  // WindowText
                                  QColor("#3e3e3e"),  // Button
@@ -125,50 +128,66 @@ ViewManager::~ViewManager()
 {
 }
 
-unsigned long
+sv_frame_t
 ViewManager::getGlobalCentreFrame() const
 {
 #ifdef DEBUG_VIEW_MANAGER
-    std::cerr << "ViewManager::getGlobalCentreFrame: returning " << m_globalCentreFrame << std::endl;
+    cerr << "ViewManager::getGlobalCentreFrame: returning " << m_globalCentreFrame << endl;
 #endif
     return m_globalCentreFrame;
 }
 
 void
-ViewManager::setGlobalCentreFrame(unsigned long f)
+ViewManager::setGlobalCentreFrame(sv_frame_t f)
 {
 #ifdef DEBUG_VIEW_MANAGER
-    std::cerr << "ViewManager::setGlobalCentreFrame to " << f << std::endl;
+    cerr << "ViewManager::setGlobalCentreFrame to " << f << endl;
 #endif
     m_globalCentreFrame = f;
     emit globalCentreFrameChanged(f);
 }
 
-unsigned long
+int
 ViewManager::getGlobalZoom() const
 {
 #ifdef DEBUG_VIEW_MANAGER
-    std::cerr << "ViewManager::getGlobalZoom: returning " << m_globalZoom << std::endl;
+    cerr << "ViewManager::getGlobalZoom: returning " << m_globalZoom << endl;
 #endif
     return m_globalZoom;
 }
 
-unsigned long
+sv_frame_t
 ViewManager::getPlaybackFrame() const
 {
-    if (m_playSource && m_playSource->isPlaying()) {
+    if (isRecording()) {
+        m_playbackFrame = m_recordTarget->getRecordDuration();
+#ifdef DEBUG_VIEW_MANAGER
+        cout << "ViewManager::getPlaybackFrame(recording) -> " << m_playbackFrame << endl;
+#endif
+    } else if (isPlaying()) {
 	m_playbackFrame = m_playSource->getCurrentPlayingFrame();
+#ifdef DEBUG_VIEW_MANAGER
+        cout << "ViewManager::getPlaybackFrame(playing) -> " << m_playbackFrame << endl;
+#endif
+    } else {
+#ifdef DEBUG_VIEW_MANAGER
+        cout << "ViewManager::getPlaybackFrame(not playing) -> " << m_playbackFrame << endl;
+#endif
     }
     return m_playbackFrame;
 }
 
 void
-ViewManager::setPlaybackFrame(unsigned long f)
+ViewManager::setPlaybackFrame(sv_frame_t f)
 {
+#ifdef DEBUG_VIEW_MANAGER
+    cerr << "ViewManager::setPlaybackFrame(" << f << ")" << endl;
+#endif
+    if (f < 0) f = 0;
     if (m_playbackFrame != f) {
 	m_playbackFrame = f;
 	emit playbackFrameChanged(f);
-	if (m_playSource && m_playSource->isPlaying()) {
+	if (isPlaying()) {
 	    m_playSource->play(f);
 	}
     }
@@ -186,18 +205,38 @@ ViewManager::setPlaybackModel(Model *model)
     m_playbackModel = model;
 }
 
-size_t
-ViewManager::alignPlaybackFrameToReference(size_t frame) const
+sv_frame_t
+ViewManager::alignPlaybackFrameToReference(sv_frame_t frame) const
 {
-    if (!m_playbackModel) return frame;
-    else return m_playbackModel->alignToReference(frame);
+#ifdef DEBUG_VIEW_MANAGER
+    cerr << "ViewManager::alignPlaybackFrameToReference(" << frame << "): playback model is " << m_playbackModel << endl;
+#endif
+    if (!m_playbackModel) {
+        return frame;
+    } else {
+        sv_frame_t f = m_playbackModel->alignToReference(frame);
+#ifdef DEBUG_VIEW_MANAGER
+        cerr << "aligned frame = " << f << endl;
+#endif
+        return f;
+    }
 }
 
-size_t
-ViewManager::alignReferenceToPlaybackFrame(size_t frame) const
+sv_frame_t
+ViewManager::alignReferenceToPlaybackFrame(sv_frame_t frame) const
 {
-    if (!m_playbackModel) return frame;
-    else return m_playbackModel->alignFromReference(frame);
+#ifdef DEBUG_VIEW_MANAGER
+    cerr << "ViewManager::alignReferenceToPlaybackFrame(" << frame << "): playback model is " << m_playbackModel << endl;
+#endif
+    if (!m_playbackModel) {
+        return frame;
+    } else {
+        sv_frame_t f = m_playbackModel->alignFromReference(frame);
+#ifdef DEBUG_VIEW_MANAGER
+        cerr << "aligned frame = " << f << endl;
+#endif
+        return f;
+    }
 }
 
 bool
@@ -257,6 +296,14 @@ ViewManager::addSelection(const Selection &selection)
     setSelections(ms);
 }
 
+void
+ViewManager::addSelectionQuietly(const Selection &selection)
+{
+    MultiSelection ms(m_selections);
+    ms.addSelection(selection);
+    setSelections(ms, true);
+}
+
 void
 ViewManager::removeSelection(const Selection &selection)
 {
@@ -274,15 +321,18 @@ ViewManager::clearSelections()
 }
 
 void
-ViewManager::setSelections(const MultiSelection &ms)
+ViewManager::setSelections(const MultiSelection &ms, bool quietly)
 {
     if (m_selections.getSelections() == ms.getSelections()) return;
     SetSelectionCommand *command = new SetSelectionCommand(this, ms);
     CommandHistory::getInstance()->addCommand(command);
+    if (!quietly) {
+        emit selectionChangedByUser();
+    }
 }
 
-size_t
-ViewManager::constrainFrameToSelection(size_t frame) const
+sv_frame_t
+ViewManager::constrainFrameToSelection(sv_frame_t frame) const
 {
     MultiSelection::SelectionList sl = getSelections();
     if (sl.empty()) return frame;
@@ -341,7 +391,7 @@ ViewManager::SetSelectionCommand::getName() const
 }
 
 Selection
-ViewManager::getContainingSelection(size_t frame, bool defaultToFollowing) const
+ViewManager::getContainingSelection(sv_frame_t frame, bool defaultToFollowing) const
 {
     return m_selections.getContainingSelection(frame, defaultToFollowing);
 }
@@ -360,9 +410,32 @@ ViewManager::setToolMode(ToolMode mode)
     case DrawMode: emit activity(tr("Enter Draw mode")); break;
     case EraseMode: emit activity(tr("Enter Erase mode")); break;
     case MeasureMode: emit activity(tr("Enter Measure mode")); break;
+    case NoteEditMode: emit activity(tr("Enter NoteEdit mode")); break; // GF: NoteEditMode activity (I'm not yet certain why we need to emit this.)
     };
 }
 
+ViewManager::ToolMode
+ViewManager::getToolModeFor(const View *v) const
+{
+    if (m_toolModeOverrides.find(v) == m_toolModeOverrides.end()) {
+        return getToolMode();
+    } else {
+        return m_toolModeOverrides.find(v)->second;
+    }
+}
+
+void
+ViewManager::setToolModeFor(const View *v, ToolMode mode)
+{
+    m_toolModeOverrides[v] = mode;
+}
+
+void
+ViewManager::clearToolModeOverrides()
+{
+    m_toolModeOverrides.clear();
+}
+
 void
 ViewManager::setPlayLoopMode(bool mode)
 {
@@ -423,7 +496,7 @@ ViewManager::setAlignMode(bool mode)
     }
 }
 
-size_t 
+sv_samplerate_t 
 ViewManager::getPlaybackSampleRate() const
 {
     if (m_playSource) {
@@ -432,11 +505,11 @@ ViewManager::getPlaybackSampleRate() const
     return 0;
 }
 
-size_t
-ViewManager::getOutputSampleRate() const
+sv_samplerate_t
+ViewManager::getDeviceSampleRate() const
 {
     if (m_playSource) {
-	return m_playSource->getTargetSampleRate();
+	return m_playSource->getDeviceSampleRate();
     }
     return 0;
 }
@@ -450,21 +523,63 @@ ViewManager::setAudioPlaySource(AudioPlaySource *source)
     m_playSource = source;
 }
 
+void
+ViewManager::setAudioRecordTarget(AudioRecordTarget *target)
+{
+    if (!m_recordTarget) {
+	QTimer::singleShot(100, this, SLOT(checkPlayStatus()));
+    }
+    m_recordTarget = target;
+}
+
 void
 ViewManager::playStatusChanged(bool /* playing */)
 {
+#ifdef DEBUG_VIEW_MANAGER
+    cerr << "ViewManager::playStatusChanged" << endl;
+#endif
+    checkPlayStatus();
+}
+
+void
+ViewManager::recordStatusChanged(bool /* recording */)
+{
+#ifdef DEBUG_VIEW_MANAGER
+    cerr << "ViewManager::recordStatusChanged" << endl;
+#endif
     checkPlayStatus();
 }
 
 void
 ViewManager::checkPlayStatus()
 {
-    if (m_playSource && m_playSource->isPlaying()) {
+    if (isRecording()) {
+
+	float left = 0, right = 0;
+	if (m_recordTarget->getInputLevels(left, right)) {
+	    if (left != m_lastLeft || right != m_lastRight) {
+		emit monitoringLevelsChanged(left, right);
+		m_lastLeft = left;
+		m_lastRight = right;
+	    }
+	}
+
+	m_playbackFrame = m_recordTarget->getRecordDuration();
+
+#ifdef DEBUG_VIEW_MANAGER
+	cerr << "ViewManager::checkPlayStatus: Recording, frame " << m_playbackFrame << ", levels " << m_lastLeft << "," << m_lastRight << endl;
+#endif
+
+	emit playbackFrameChanged(m_playbackFrame);
+
+	QTimer::singleShot(500, this, SLOT(checkPlayStatus()));
+
+    } else if (isPlaying()) {
 
 	float left = 0, right = 0;
 	if (m_playSource->getOutputLevels(left, right)) {
 	    if (left != m_lastLeft || right != m_lastRight) {
-		emit outputLevelsChanged(left, right);
+		emit monitoringLevelsChanged(left, right);
 		m_lastLeft = left;
 		m_lastRight = right;
 	    }
@@ -473,7 +588,7 @@ ViewManager::checkPlayStatus()
 	m_playbackFrame = m_playSource->getCurrentPlayingFrame();
 
 #ifdef DEBUG_VIEW_MANAGER
-	std::cerr << "ViewManager::checkPlayStatus: Playing, frame " << m_playbackFrame << ", levels " << m_lastLeft << "," << m_lastRight << std::endl;
+	cerr << "ViewManager::checkPlayStatus: Playing, frame " << m_playbackFrame << ", levels " << m_lastLeft << "," << m_lastRight << endl;
 #endif
 
 	emit playbackFrameChanged(m_playbackFrame);
@@ -482,16 +597,14 @@ ViewManager::checkPlayStatus()
 
     } else {
 
-	QTimer::singleShot(100, this, SLOT(checkPlayStatus()));
-	
 	if (m_lastLeft != 0.0 || m_lastRight != 0.0) {
-	    emit outputLevelsChanged(0.0, 0.0);
+	    emit monitoringLevelsChanged(0.0, 0.0);
 	    m_lastLeft = 0.0;
 	    m_lastRight = 0.0;
 	}
 
 #ifdef DEBUG_VIEW_MANAGER
-//	std::cerr << "ViewManager::checkPlayStatus: Not playing" << std::endl;
+	cerr << "ViewManager::checkPlayStatus: Not playing or recording" << endl;
 #endif
     }
 }
@@ -502,14 +615,20 @@ ViewManager::isPlaying() const
     return m_playSource && m_playSource->isPlaying();
 }
 
+bool
+ViewManager::isRecording() const
+{
+    return m_recordTarget && m_recordTarget->isRecording();
+}
+
 void
-ViewManager::viewCentreFrameChanged(unsigned long f, bool locked,
+ViewManager::viewCentreFrameChanged(sv_frame_t f, bool locked,
                                     PlaybackFollowMode mode)
 {
     View *v = dynamic_cast<View *>(sender());
 
 #ifdef DEBUG_VIEW_MANAGER
-    std::cerr << "ViewManager::viewCentreFrameChanged(" << f << ", " << locked << ", " << mode << "), view is " << v << std::endl;
+    cerr << "ViewManager::viewCentreFrameChanged(" << f << ", " << locked << ", " << mode << "), view is " << v << endl;
 #endif
 
     if (locked) {
@@ -527,28 +646,35 @@ ViewManager::viewCentreFrameChanged(unsigned long f, bool locked,
         }
     }
 
-    if (mode == PlaybackIgnore) {
-        return;
+    if (mode == PlaybackScrollPageWithCentre ||
+        mode == PlaybackScrollContinuous) {
+        seek(f);
     }
-
-    seek(f);
 }
 
 void
-ViewManager::seek(unsigned long f)
+ViewManager::seek(sv_frame_t f)
 {
 #ifdef DEBUG_VIEW_MANAGER 
-    std::cerr << "ViewManager::seek(" << f << ")" << std::endl;
+    cerr << "ViewManager::seek(" << f << ")" << endl;
 #endif
 
-    if (m_playSource && m_playSource->isPlaying()) {
-	unsigned long playFrame = m_playSource->getCurrentPlayingFrame();
-	unsigned long diff = std::max(f, playFrame) - std::min(f, playFrame);
+    if (isRecording()) {
+        // ignore
+#ifdef DEBUG_VIEW_MANAGER
+        cerr << "ViewManager::seek: Ignoring during recording" << endl;
+#endif
+        return;
+    }
+    
+    if (isPlaying()) {
+	sv_frame_t playFrame = m_playSource->getCurrentPlayingFrame();
+	sv_frame_t diff = std::max(f, playFrame) - std::min(f, playFrame);
 	if (diff > 20000) {
 	    m_playbackFrame = f;
 	    m_playSource->play(f);
 #ifdef DEBUG_VIEW_MANAGER 
-	    std::cerr << "ViewManager::considerSeek: reseeking from " << playFrame << " to " << f << std::endl;
+	    cerr << "ViewManager::seek: reseeking from " << playFrame << " to " << f << endl;
 #endif
             emit playbackFrameChanged(f);
 	}
@@ -561,7 +687,7 @@ ViewManager::seek(unsigned long f)
 }
 
 void
-ViewManager::viewZoomLevelChanged(unsigned long z, bool locked)
+ViewManager::viewZoomLevelChanged(int z, bool locked)
 {
     View *v = dynamic_cast<View *>(sender());
 
@@ -577,7 +703,7 @@ ViewManager::viewZoomLevelChanged(unsigned long z, bool locked)
     }
 
 #ifdef DEBUG_VIEW_MANAGER 
-    std::cerr << "ViewManager::viewZoomLevelChanged(" << v << ", " << z << ", " << locked << ")" << std::endl;
+    cerr << "ViewManager::viewZoomLevelChanged(" << v << ", " << z << ", " << locked << ")" << endl;
 #endif
 
     emit viewZoomLevelChanged(v, z, locked);
@@ -645,11 +771,13 @@ ViewManager::setGlobalDarkBackground(bool dark)
         m_lightPalette = QApplication::palette();
     }
 
+#ifndef Q_OS_MAC
     if (dark) {
         QApplication::setPalette(m_darkPalette);
     } else {
         QApplication::setPalette(m_lightPalette);
     }
+#endif
 }
 
 bool
@@ -663,3 +791,27 @@ ViewManager::getGlobalDarkBackground() const
     return dark;
 }
 
+int
+ViewManager::scalePixelSize(int pixels)
+{
+    static double ratio = 0.0;
+    if (ratio == 0.0) {
+        double baseEm;
+#ifdef Q_OS_MAC
+        baseEm = 17.0;
+#else
+        baseEm = 15.0;
+#endif
+        double em = QFontMetrics(QFont()).height();
+        ratio = em / baseEm;
+
+        SVDEBUG << "ViewManager::scalePixelSize: ratio is " << ratio
+                << " (em = " << em << ")" << endl;
+    }
+
+    int scaled = int(pixels * ratio + 0.5);
+//    cerr << "scaledSize: " << pixels << " -> " << scaled << " at ratio " << ratio << endl;
+    if (pixels != 0 && scaled == 0) scaled = 1;
+    return scaled;
+}
+
