@@ -105,7 +105,7 @@ void
 TipDialog::previous()
 {
     if (--m_tipNumber < 0) {
-        m_tipNumber = m_tips.size() - 1;
+        m_tipNumber = int(m_tips.size()) - 1;
     }
 
     showTip();
@@ -146,7 +146,7 @@ void
 TipDialog::showTip()
 {
     if (m_tipNumber < int(m_tips.size())) {
-        std::cerr << "Tip " << m_tipNumber << " is: " << m_tips[m_tipNumber] << std::endl;
+        cerr << "Tip " << m_tipNumber << " is: " << m_tips[m_tipNumber] << endl;
         m_label->setText(m_tips[m_tipNumber]);
     } else {
         accept();
@@ -196,24 +196,24 @@ TipDialog::TipFileParser::startElement(const QString &, const QString &,
         if (caption != "") m_dialog->m_caption = caption;
     } else if (name == "tip") {
         if (m_inTip) {
-            std::cerr << "WARNING: TipFileParser: nested <tip> elements" << std::endl;
+            cerr << "WARNING: TipFileParser: nested <tip> elements" << endl;
         }
         m_inTip = true;
     } else if (name == "text") {
         if (m_inTip) {
             m_inText = true;
-            std::cerr << "TipFileParser: adding new tip" << std::endl;
+            cerr << "TipFileParser: adding new tip" << endl;
             m_dialog->m_tips.push_back("");
         } else {
-            std::cerr << "WARNING: TipFileParser: <text> outside <tip> element" << std::endl;
+            cerr << "WARNING: TipFileParser: <text> outside <tip> element" << endl;
         }
     } else if (name == "html") {
         if (m_inTip) {
             m_inHtml = true;
-            std::cerr << "TipFileParser: adding new tip" << std::endl;
+            cerr << "TipFileParser: adding new tip" << endl;
             m_dialog->m_tips.push_back("");
         } else {
-            std::cerr << "WARNING: TipFileParser: <html> outside <tip> element" << std::endl;
+            cerr << "WARNING: TipFileParser: <html> outside <tip> element" << endl;
         }
     } else if (m_inHtml) {
         m_dialog->m_tips[m_dialog->m_tips.size()-1] += "<" + qName;
@@ -236,21 +236,21 @@ TipDialog::TipFileParser::endElement(const QString &, const QString &,
 
     if (name == "text") {
         if (!m_inText) {
-            std::cerr << "WARNING: TipFileParser: </text> without <text>" << std::endl;
+            cerr << "WARNING: TipFileParser: </text> without <text>" << endl;
         }
         m_inText = false;
     } else if (name == "html") {
         if (!m_inHtml) {
-            std::cerr << "WARNING: TipFileParser: </html> without <html>" << std::endl;
+            cerr << "WARNING: TipFileParser: </html> without <html>" << endl;
         }
         m_inHtml = false;
     } else if (name == "tip") {
         if (m_inText) {
-            std::cerr << "WARNING: TipFileParser: <text> without </text>" << std::endl;
+            cerr << "WARNING: TipFileParser: <text> without </text>" << endl;
         } else if (m_inHtml) {
-            std::cerr << "WARNING: TipFileParser: <html> without </html>" << std::endl;
+            cerr << "WARNING: TipFileParser: <html> without </html>" << endl;
         } else if (!m_inTip) {
-            std::cerr << "WARNING: TipFileParser: </tip> without <tip>" << std::endl;
+            cerr << "WARNING: TipFileParser: </tip> without <tip>" << endl;
         }
         m_inTip = false;
     } else if (m_inHtml) {
@@ -280,7 +280,7 @@ TipDialog::TipFileParser::error(const QXmlParseException &exception)
 	.arg(exception.message())
 	.arg(exception.lineNumber())
 	.arg(exception.columnNumber());
-    std::cerr << errorString << std::endl;
+    cerr << errorString << endl;
     return QXmlDefaultHandler::error(exception);
 }
 
@@ -292,6 +292,6 @@ TipDialog::TipFileParser::fatalError(const QXmlParseException &exception)
 	.arg(exception.message())
 	.arg(exception.lineNumber())
 	.arg(exception.columnNumber());
-    std::cerr << errorString << std::endl;
+    cerr << errorString << endl;
     return QXmlDefaultHandler::fatalError(exception);
 }
